#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Azure auth method module."""
import logging

from hvac import exceptions, utils
from hvac.api.vault_api_base import VaultApiBase
from hvac.constants.azure import VALID_ENVIRONMENTS

DEFAULT_MOUNT_POINT = "azure"
logger = logging.getLogger(__name__)


class Azure(VaultApiBase):
    """Azure Auth Method (API).

    Reference: https://www.vaultproject.io/api/auth/azure/index.html
    """

    def configure(
        self,
        tenant_id,
        resource,
        environment=None,
        client_id=None,
        client_secret=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Configure the credentials required for the plugin to perform API calls to Azure.

        These credentials will be used to query the metadata about the virtual machine.

        Supported methods:
            POST: /auth/{mount_point}/config. Produces: 204 (empty body)

        :param tenant_id: The tenant id for the Azure Active Directory organization.
        :type tenant_id: str | unicode
        :param resource: The configured URL for the application registered in Azure Active Directory.
        :type resource: str | unicode
        :param environment: The Azure cloud environment. Valid values: AzurePublicCloud, AzureUSGovernmentCloud,
            AzureChinaCloud, AzureGermanCloud.
        :type environment: str | unicode
        :param client_id: The client id for credentials to query the Azure APIs.  Currently read permissions to query
            compute resources are required.
        :type client_id: str | unicode
        :param client_secret: The client secret for credentials to query the Azure APIs.
        :type client_secret: str | unicode
        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        if environment is not None and environment not in VALID_ENVIRONMENTS:
            error_msg = 'invalid environment argument provided: "{arg}"; supported environments: "{environments}"'
            raise exceptions.ParamValidationError(
                error_msg.format(
                    arg=environment,
                    environments=",".join(VALID_ENVIRONMENTS),
                )
            )
        params = {
            "tenant_id": tenant_id,
            "resource": resource,
        }
        params.update(
            utils.remove_nones(
                {
                    "environment": environment,
                    "client_id": client_id,
                    "client_secret": client_secret,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_config(self, mount_point=DEFAULT_MOUNT_POINT):
        """Return the previously configured config, including credentials.

        Supported methods:
            GET: /auth/{mount_point}/config. Produces: 200 application/json

        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The data key from the JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        response = self._adapter.get(
            url=api_path,
        )
        return response.get("data")

    def delete_config(self, mount_point=DEFAULT_MOUNT_POINT):
        """Delete the previously configured Azure config and credentials.

        Supported methods:
            DELETE: /auth/{mount_point}/config. Produces: 204 (empty body)

        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_role(
        self,
        name,
        policies=None,
        ttl=None,
        max_ttl=None,
        period=None,
        bound_service_principal_ids=None,
        bound_group_ids=None,
        bound_locations=None,
        bound_subscription_ids=None,
        bound_resource_groups=None,
        bound_scale_sets=None,
        num_uses=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create a role in the method.

        Role types have specific entities that can perform login operations against this endpoint. Constraints specific
        to the role type must be set on the role. These are applied to the authenticated entities attempting to login.

        Supported methods:
            POST: /auth/{mount_point}/role/{name}. Produces: 204 (empty body)


        :param name: Name of the role.
        :type name: str | unicode
        :param policies: Policies to be set on tokens issued using this role.
        :type policies: str | list
        :param num_uses: Number of uses to set on a token produced by this role.
        :type num_uses: int
        :param ttl: The TTL period of tokens issued using this role in seconds.
        :type ttl: str | unicode
        :param max_ttl: The maximum allowed lifetime of tokens issued in seconds using this role.
        :type max_ttl: str | unicode
        :param period: If set, indicates that the token generated using this role should never expire. The token should
            be renewed within the duration specified by this value. At each renewal, the token's TTL will be set to the
            value of this parameter.
        :type period: str | unicode
        :param bound_service_principal_ids: The list of Service Principal IDs that login is restricted to.
        :type bound_service_principal_ids: list
        :param bound_group_ids: The list of group ids that login is restricted to.
        :type bound_group_ids: list
        :param bound_locations: The list of locations that login is restricted to.
        :type bound_locations: list
        :param bound_subscription_ids: The list of subscription IDs that login is restricted to.
        :type bound_subscription_ids: list
        :param bound_resource_groups: The list of resource groups that login is restricted to.
        :type bound_resource_groups: list
        :param bound_scale_sets: The list of scale set names that the login is restricted to.
        :type bound_scale_sets: list
        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        if policies is not None:
            if not (
                isinstance(policies, str)
                or (
                    isinstance(policies, list)
                    and all(isinstance(p, str) for p in policies)
                )
            ):
                error_msg = 'unsupported policies argument provided "{arg}" ({arg_type}), required type: str or List[str]"'
                raise exceptions.ParamValidationError(
                    error_msg.format(
                        arg=policies,
                        arg_type=type(policies),
                    )
                )
        params = utils.remove_nones(
            {
                "policies": policies,
                "ttl": ttl,
                "max_ttl": max_ttl,
                "period": period,
                "bound_service_principal_ids": bound_service_principal_ids,
                "bound_group_ids": bound_group_ids,
                "bound_locations": bound_locations,
                "bound_subscription_ids": bound_subscription_ids,
                "bound_resource_groups": bound_resource_groups,
                "bound_scale_sets": bound_scale_sets,
                "num_uses": num_uses,
            }
        )

        api_path = utils.format_url(
            "/v1/auth/{mount_point}/role/{name}", mount_point=mount_point, name=name
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Read the previously registered role configuration.

        Supported methods:
            GET: /auth/{mount_point}/role/{name}. Produces: 200 application/json


        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The "data" key from the JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/role/{name}",
            mount_point=mount_point,
            name=name,
        )
        response = self._adapter.get(
            url=api_path,
        )
        return response.get("data")

    def list_roles(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all the roles that are registered with the plugin.

        Supported methods:
            LIST: /auth/{mount_point}/role. Produces: 200 application/json


        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The "data" key from the JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/role", mount_point=mount_point
        )
        response = self._adapter.list(url=api_path)
        return response.get("data")

    def delete_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete the previously registered role.

        Supported methods:
            DELETE: /auth/{mount_point}/role/{name}. Produces: 204 (empty body)


        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/role/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def login(
        self,
        role,
        jwt,
        subscription_id=None,
        resource_group_name=None,
        vm_name=None,
        vmss_name=None,
        use_token=True,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Fetch a token.

        This endpoint takes a signed JSON Web Token (JWT) and a role name for some entity. It verifies the JWT signature
        to authenticate that entity and then authorizes the entity for the given role.

        Supported methods:
            POST: /auth/{mount_point}/login. Produces: 200 application/json


        :param role: Name of the role against which the login is being attempted.
        :type role: str | unicode
        :param jwt: Signed JSON Web Token (JWT) from Azure MSI.
        :type jwt: str | unicode
        :param subscription_id: The subscription ID for the machine that generated the MSI token. This information can
            be obtained through instance metadata.
        :type subscription_id: str | unicode
        :param resource_group_name: The resource group for the machine that generated the MSI token. This information
            can be obtained through instance metadata.
        :type resource_group_name: str | unicode
        :param vm_name: The virtual machine name for the machine that generated the MSI token. This information can be
            obtained through instance metadata.  If vmss_name is provided, this value is ignored.
        :type vm_name: str | unicode
        :param vmss_name: The virtual machine scale set name for the machine that generated the MSI token. This
            information can be obtained through instance metadata.
        :type vmss_name: str | unicode
        :param use_token: if True, uses the token in the response received from the auth request to set the "token"
            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapater Client attribute.
        :type use_token: bool
        :param mount_point: The "path" the azure auth method was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {
            "role": role,
            "jwt": jwt,
        }
        params.update(
            utils.remove_nones(
                {
                    "subscription_id": subscription_id,
                    "resource_group_name": resource_group_name,
                    "vm_name": vm_name,
                    "vmss_name": vmss_name,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/login", mount_point=mount_point
        )
        return self._adapter.login(
            url=api_path,
            use_token=use_token,
            json=params,
        )



"""Base class used by all hvac api "category" classes."""
import logging
from abc import ABCMeta, abstractproperty

from hvac.api.vault_api_base import VaultApiBase

logger = logging.getLogger(__name__)


class VaultApiCategory(VaultApiBase):
    """Base class for API categories."""

    __metaclass__ = ABCMeta

    def __init__(self, adapter):
        """API Category class constructor.

        :param adapter: Instance of :py:class:`hvac.adapters.Adapter`; used for performing HTTP requests.
        :type adapter: hvac.adapters.Adapter
        """
        self._adapter = adapter
        self.implemented_class_names = []
        for implemented_class in self.implemented_classes:
            class_name = implemented_class.__name__.lower()
            self.implemented_class_names.append(class_name)
            auth_method_instance = implemented_class(adapter=adapter)
            setattr(self, self.get_private_attr_name(class_name), auth_method_instance)

        super(VaultApiCategory, self).__init__(adapter=adapter)

    def __getattr__(self, item):
        """Get an instance of an class instance in this category where available.

        :param item: Name of the class being requested.
        :type item: str | unicode
        :return: The requested class instance where available.
        :rtype: hvac.api.VaultApiBase
        """
        if item in self.implemented_class_names:
            private_attr_name = self.get_private_attr_name(item)
            return getattr(self, private_attr_name)
        if item in [u.lower() for u in self.unimplemented_classes]:
            raise NotImplementedError(
                '"%s" auth method class not currently implemented.' % item
            )
        raise AttributeError

    @property
    def adapter(self):
        """Retrieve the adapter instance under the "_adapter" property in use by this class.

        :return: The adapter instance in use by this class.
        :rtype: hvac.adapters.Adapter
        """
        return self._adapter

    @adapter.setter
    def adapter(self, adapter):
        """Sets the adapter instance under the "_adapter" property in use by this class.

        Also sets the adapter property for all implemented classes under this category.

        :param adapter: New adapter instance to set for this class and all implemented classes under this category.
        :type adapter: hvac.adapters.Adapter
        """
        self._adapter = adapter
        for implemented_class in self.implemented_classes:
            class_name = implemented_class.__name__.lower()
            getattr(self, self.get_private_attr_name(class_name)).adapter = adapter

    @abstractproperty
    def implemented_classes(self):
        """List of implemented classes under this category.

        :return: List of implemented classes under this category.
        :rtype: List[hvac.api.VaultApiBase]
        """
        raise NotImplementedError

    @property
    def unimplemented_classes(self):
        """List of known unimplemented classes under this category.

        :return: List of known unimplemented classes under this category.
        :rtype: List[str]
        """
        raise NotImplementedError

    @staticmethod
    def get_private_attr_name(class_name):
        """Helper method to prepend a leading underscore to a provided class name.

        :param class_name: Name of a class under this category.
        :type class_name: str|unicode
        :return: The private attribute label for the provided class.
        :rtype: str
        """
        private_attr_name = "_{class_name}".format(class_name=class_name)
        return private_attr_name



"""Kv secret backend methods module."""

import logging

from hvac.api.secrets_engines import kv_v1, kv_v2
from hvac.api.vault_api_base import VaultApiBase

logger = logging.getLogger(__name__)


class Kv(VaultApiBase):
    """Class containing methods for the key/value secrets_engines backend API routes.
    Reference: https://www.vaultproject.io/docs/secrets/kv/index.html

    """

    allowed_kv_versions = ["1", "2"]

    def __init__(self, adapter, default_kv_version="2"):
        """Create a new Kv instance.

        :param adapter: Instance of :py:class:`hvac.adapters.Adapter`; used for performing HTTP requests.
        :type adapter: hvac.adapters.Adapter
        :param default_kv_version: KV version number (e.g., '1') to use as the default when accessing attributes/methods
            under this class.
        :type default_kv_version: str | unicode
        """
        super(Kv, self).__init__(adapter=adapter)
        self._default_kv_version = default_kv_version

        self._kv_v1 = kv_v1.KvV1(adapter=self._adapter)
        self._kv_v2 = kv_v2.KvV2(adapter=self._adapter)

    @property
    def v1(self):
        """Accessor for kv version 1 class / method. Provided via the :py:class:`hvac.api.secrets_engines.kv_v1.KvV1` class.

        :return: This Kv instance's associated KvV1 instance.
        :rtype: hvac.api.secrets_engines.kv_v1.KvV1
        """
        return self._kv_v1

    @property
    def v2(self):
        """Accessor for kv version 2 class / method. Provided via the :py:class:`hvac.api.secrets_engines.kv_v2.KvV2` class.

        :return: This Kv instance's associated KvV2 instance.
        :rtype: hvac.api.secrets_engines.kv_v2.KvV2
        """
        return self._kv_v2

    @property
    def default_kv_version(self):
        return self._default_kv_version

    @default_kv_version.setter
    def default_kv_version(self, default_kv_version):
        if str(default_kv_version) not in self.allowed_kv_versions:
            error_message = 'Invalid "default_kv_version"; "{allowed}" allowed, "{provided}" provided'.format(
                allowed=",".join(self.allowed_kv_versions), provided=default_kv_version
            )
            raise ValueError(error_message)
        self._default_kv_version = str(default_kv_version)

    def __getattr__(self, item):
        """Overridden magic method used to direct method calls to the appropriate KV version's hvac class.

        :param item: Name of the attribute/method being accessed
        :type item: str | unicode
        :return: The selected secrets_engines class corresponding to this instance's default_kv_version setting
        :rtype: hvac.api.vault_api_base.VaultApiBase
        """
        if self.default_kv_version == "1":
            return getattr(self._kv_v1, item)
        elif self.default_kv_version == "2":
            return getattr(self._kv_v2, item)

        raise AttributeError



"""Collection of classes for various Vault auth methods."""

import warnings

from hvac.api.auth_methods.approle import AppRole
from hvac.api.auth_methods.azure import Azure
from hvac.api.auth_methods.gcp import Gcp
from hvac.api.auth_methods.github import Github
from hvac.api.auth_methods.jwt import JWT
from hvac.api.auth_methods.kubernetes import Kubernetes
from hvac.api.auth_methods.ldap import Ldap
from hvac.api.auth_methods.userpass import Userpass
from hvac.api.auth_methods.mfa import Mfa
from hvac.api.auth_methods.oidc import OIDC
from hvac.api.auth_methods.okta import Okta
from hvac.api.auth_methods.radius import Radius
from hvac.api.auth_methods.token import Token
from hvac.api.auth_methods.aws import Aws
from hvac.api.auth_methods.cert import Cert
from hvac.api.vault_api_category import VaultApiCategory
from hvac.utils import generate_method_deprecation_message

__all__ = (
    "AuthMethods",
    "AppRole",
    "Azure",
    "Gcp",
    "Github",
    "JWT",
    "Kubernetes",
    "Ldap",
    "Userpass",
    "Mfa",
    "OIDC",
    "Okta",
    "Radius",
    "Token",
    "Aws",
    "Cert",
)


class AuthMethods(VaultApiCategory):
    """Auth Methods."""

    implemented_classes = [
        AppRole,
        Azure,
        Github,
        Gcp,
        JWT,
        Kubernetes,
        Ldap,
        Userpass,
        Mfa,
        OIDC,
        Okta,
        Radius,
        Token,
        Aws,
        Cert,
    ]
    unimplemented_classes = [
        "AppId",
        "AliCloud",
    ]

    def __call__(self, *args, **kwargs):
        """Implement callable magic method for backwards compatibility.

        Older versions of hvac.Client had an auth method that has now been replaced with an "auth" property pointing to
        this class.
        """
        deprecation_message = generate_method_deprecation_message(
            to_be_removed_in_version="0.9.0",
            old_method_name="auth",
            method_name="login",
            module_name="adapters.Request",
        )
        warnings.simplefilter("always", DeprecationWarning)
        warnings.warn(
            message=deprecation_message,
            category=DeprecationWarning,
            stacklevel=2,
        )
        warnings.simplefilter("default", DeprecationWarning)

        return self._adapter.login(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the AWS auth method and/or secrets engine."""

DEFAULT_MOUNT_POINT = "aws"
ALLOWED_CREDS_ENDPOINTS = ["creds", "sts"]
ALLOWED_CREDS_TYPES = ["iam_user", "assumed_role", "federation_token"]
ALLOWED_IAM_ALIAS_TYPES = ["role_id", "unique_id", "full_arn"]
ALLOWED_EC2_ALIAS_TYPES = ["role_id", "instance_id", "image_id"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the GCP auth method and/or secrets engine."""

DEFAULT_MOUNT_POINT = "gcp"
ALLOWED_ROLE_TYPES = ["iam", "gce"]
ALLOWED_SECRETS_TYPES = ["access_token", "service_account_key"]
SERVICE_ACCOUNT_KEY_ALGORITHMS = [
    "KEY_ALG_UNSPECIFIED",
    "KEY_ALG_RSA_1024",
    "KEY_ALG_RSA_2048",
]
SERVICE_ACCOUNT_KEY_TYPES = [
    "TYPE_UNSPECIFIED",
    "TYPE_PKCS12_FILE",
    "TYPE_GOOGLE_CREDENTIALS_FILE",
]
GCP_CERTS_ENDPOINT = "https://www.googleapis.com/oauth2/v3/certs"



from __future__ import unicode_literals

import json
import os
from base64 import b64encode

from hvac import adapters, api, aws_utils, exceptions, utils
from hvac.constants.client import (
    DEFAULT_URL,
    DEPRECATED_PROPERTIES,
    VAULT_CACERT,
    VAULT_CAPATH,
    VAULT_CLIENT_CERT,
    VAULT_CLIENT_KEY,
)
from hvac.utils import generate_property_deprecation_message

try:
    import hcl

    has_hcl_parser = True
except ImportError:
    has_hcl_parser = False


class Client(object):
    """The hvac Client class for HashiCorp's Vault."""

    def __init__(
        self,
        url=None,
        token=None,
        cert=None,
        verify=True,
        timeout=30,
        proxies=None,
        allow_redirects=True,
        session=None,
        adapter=adapters.JSONAdapter,
        namespace=None,
        **kwargs
    ):
        """Creates a new hvac client instance.

        :param url: Base URL for the Vault instance being addressed.
        :type url: str
        :param token: Authentication token to include in requests sent to Vault.
        :type token: str
        :param cert: Certificates for use in requests sent to the Vault instance. This should be a tuple with the
            certificate and then key.
        :type cert: tuple
        :param verify: Either a boolean to indicate whether TLS verification should be performed when sending requests to Vault,
            or a string pointing at the CA bundle to use for verification. See http://docs.python-requests.org/en/master/user/advanced/#ssl-cert-verification.
        :type verify: Union[bool,str]
        :param timeout: The timeout value for requests sent to Vault.
        :type timeout: int
        :param proxies: Proxies to use when performing requests.
            See: http://docs.python-requests.org/en/master/user/advanced/#proxies
        :type proxies: dict
        :param allow_redirects: Whether to follow redirects when sending requests to Vault.
        :type allow_redirects: bool
        :param session: Optional session object to use when performing request.
        :type session: request.Session
        :param adapter: Optional class to be used for performing requests. If none is provided, defaults to
            hvac.adapters.JSONRequest
        :type adapter: hvac.adapters.Adapter
        :param kwargs: Additional parameters to pass to the adapter constructor.
        :type kwargs: dict
        :param namespace: Optional Vault Namespace.
        :type namespace: str
        """

        token = token if token is not None else utils.get_token_from_env()
        url = url if url else os.getenv("VAULT_ADDR", DEFAULT_URL)

        if cert is not None and VAULT_CLIENT_CERT:
            cert = "\n".join(
                [
                    VAULT_CLIENT_CERT,
                    VAULT_CLIENT_KEY,
                ]
            )

        # Consider related CA env vars _only if_ no argument is passed in under the
        # `verify` parameter.
        if verify is not None:
            # Reference: https://www.vaultproject.io/docs/commands#vault_cacert
            # Note: "[VAULT_CACERT] takes precedence over VAULT_CAPATH." and thus we
            # check for VAULT_CAPATH _first_.
            if VAULT_CAPATH:
                verify = VAULT_CAPATH
            if VAULT_CACERT:
                verify = VAULT_CACERT

        self._adapter = adapter(
            base_uri=url,
            token=token,
            cert=cert,
            verify=verify,
            timeout=timeout,
            proxies=proxies,
            allow_redirects=allow_redirects,
            session=session,
            namespace=namespace,
            **kwargs
        )

        # Instantiate API classes to be exposed as properties on this class starting with auth method classes.
        self._auth = api.AuthMethods(adapter=self._adapter)
        self._secrets = api.SecretsEngines(adapter=self._adapter)
        self._sys = api.SystemBackend(adapter=self._adapter)

    def __getattr__(self, name):
        return utils.getattr_with_deprecated_properties(
            obj=self, item=name, deprecated_properties=DEPRECATED_PROPERTIES
        )

    @property
    def adapter(self):
        return self._adapter

    @adapter.setter
    def adapter(self, adapter):
        self._adapter = adapter

    @property
    def url(self):
        return self._adapter.base_uri

    @url.setter
    def url(self, url):
        self._adapter.base_uri = url

    @property
    def token(self):
        return self._adapter.token

    @token.setter
    def token(self, token):
        self._adapter.token = token

    @property
    def session(self):
        return self._adapter.session

    @session.setter
    def session(self, session):
        self._adapter.session = session

    @property
    def allow_redirects(self):
        return self._adapter.allow_redirects

    @allow_redirects.setter
    def allow_redirects(self, allow_redirects):
        self._adapter.allow_redirects = allow_redirects

    @property
    def auth(self):
        """Accessor for the Client instance's auth methods. Provided via the :py:class:`hvac.api.AuthMethods` class.
        :return: This Client instance's associated Auth instance.
        :rtype: hvac.api.AuthMethods
        """
        return self._auth

    @property
    def secrets(self):
        """Accessor for the Client instance's secrets engines. Provided via the :py:class:`hvac.api.SecretsEngines` class.

        :return: This Client instance's associated SecretsEngines instance.
        :rtype: hvac.api.SecretsEngines
        """
        return self._secrets

    @property
    def sys(self):
        """Accessor for the Client instance's system backend methods.
        Provided via the :py:class:`hvac.api.SystemBackend` class.

        :return: This Client instance's associated SystemBackend instance.
        :rtype: hvac.api.SystemBackend
        """
        return self._sys

    @property
    def generate_root_status(self):
        return self.sys.read_root_generation_progress()

    @property
    def key_status(self):
        """GET /sys/key-status

        :return: Information about the current encryption key used by Vault.
        :rtype: dict
        """
        return self.sys.get_encryption_key_status()["data"]

    @property
    def rekey_status(self):
        return self.sys.read_rekey_progress()

    @property
    def ha_status(self):
        """Read the high availability status and current leader instance of Vault.

        :return: The JSON response returned by read_leader_status()
        :rtype: dict
        """
        return self.sys.read_leader_status()

    @property
    def seal_status(self):
        """Read the seal status of the Vault.

        This is an unauthenticated endpoint.

        Supported methods:
            GET: /sys/seal-status. Produces: 200 application/json

        :return: The JSON response of the request.
        :rtype: dict
        """
        return self.sys.read_seal_status()

    def read(self, path, wrap_ttl=None):
        """GET /<path>

        :param path:
        :type path:
        :param wrap_ttl:
        :type wrap_ttl:
        :return:
        :rtype:
        """
        try:
            return self._adapter.get("/v1/{0}".format(path), wrap_ttl=wrap_ttl)
        except exceptions.InvalidPath:
            return None

    def list(self, path):
        """GET /<path>?list=true

        :param path:
        :type path:
        :return:
        :rtype:
        """
        try:
            payload = {"list": True}
            return self._adapter.get("/v1/{0}".format(path), params=payload)
        except exceptions.InvalidPath:
            return None

    def write(self, path, wrap_ttl=None, **kwargs):
        """POST /<path>

        :param path:
        :type path:
        :param wrap_ttl:
        :type wrap_ttl:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        return self._adapter.post(
            "/v1/{0}".format(path), json=kwargs, wrap_ttl=wrap_ttl
        )

    def delete(self, path):
        """DELETE /<path>

        :param path:
        :type path:
        :return:
        :rtype:
        """
        self._adapter.delete("/v1/{0}".format(path))

    def get_policy(self, name, parse=False):
        """Retrieve the policy body for the named policy.

        :param name: The name of the policy to retrieve.
        :type name: str | unicode
        :param parse: Specifies whether to parse the policy body using pyhcl or not.
        :type parse: bool
        :return: The (optionally parsed) policy body for the specified policy.
        :rtype: str | dict
        """
        try:
            policy = self.sys.read_policy(name=name)["data"]["rules"]
        except exceptions.InvalidPath:
            return None

        if parse:
            if not has_hcl_parser:
                raise ImportError("pyhcl is required for policy parsing")
            policy = hcl.loads(policy)

        return policy

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.revoke_self,
    )
    def revoke_self_token(self):
        """PUT /auth/token/revoke-self

        :return:
        :rtype:
        """
        self._adapter.put("/v1/auth/token/revoke-self")

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.create,
    )
    def create_token(
        self,
        role=None,
        token_id=None,
        policies=None,
        meta=None,
        no_parent=False,
        lease=None,
        display_name=None,
        num_uses=None,
        no_default_policy=False,
        ttl=None,
        orphan=False,
        wrap_ttl=None,
        renewable=None,
        explicit_max_ttl=None,
        period=None,
        token_type=None,
    ):
        """POST /auth/token/create

        POST /auth/token/create/<role>

        POST /auth/token/create-orphan

        :param role:
        :type role:
        :param token_id:
        :type token_id:
        :param policies:
        :type policies:
        :param meta:
        :type meta:
        :param no_parent:
        :type no_parent:
        :param lease:
        :type lease:
        :param display_name:
        :type display_name:
        :param num_uses:
        :type num_uses:
        :param no_default_policy:
        :type no_default_policy:
        :param ttl:
        :type ttl:
        :param orphan:
        :type orphan:
        :param wrap_ttl:
        :type wrap_ttl:
        :param renewable:
        :type renewable:
        :param explicit_max_ttl:
        :type explicit_max_ttl:
        :param period:
        :type period:
        :param token_type:
        :type token_type:
        :return:
        :rtype:
        """
        params = {
            "id": token_id,
            "policies": policies,
            "meta": meta,
            "no_parent": no_parent,
            "display_name": display_name,
            "num_uses": num_uses,
            "no_default_policy": no_default_policy,
            "renewable": renewable,
        }

        if lease:
            params["lease"] = lease
        else:
            params["ttl"] = ttl
            params["explicit_max_ttl"] = explicit_max_ttl

        if explicit_max_ttl:
            params["explicit_max_ttl"] = explicit_max_ttl

        if period:
            params["period"] = period
        if token_type:
            params["type"] = token_type

        if orphan:
            return self._adapter.post(
                "/v1/auth/token/create-orphan", json=params, wrap_ttl=wrap_ttl
            )
        elif role:
            return self._adapter.post(
                "/v1/auth/token/create/{0}".format(role), json=params, wrap_ttl=wrap_ttl
            )
        else:
            return self._adapter.post(
                "/v1/auth/token/create", json=params, wrap_ttl=wrap_ttl
            )

    def lookup_token(self, token=None, accessor=False, wrap_ttl=None):
        """GET /auth/token/lookup/<token>

        GET /auth/token/lookup-accessor/<token-accessor>

        GET /auth/token/lookup-self

        :param token:
        :type token: str.
        :param accessor:
        :type accessor: str.
        :param wrap_ttl:
        :type wrap_ttl: int.
        :return:
        :rtype:
        """
        token_param = {
            "token": token,
        }
        accessor_param = {
            "accessor": token,
        }
        if token:
            if accessor:
                path = "/v1/auth/token/lookup-accessor"
                return self._adapter.post(path, json=accessor_param, wrap_ttl=wrap_ttl)
            else:
                path = "/v1/auth/token/lookup"
                return self._adapter.post(path, json=token_param)
        else:
            path = "/v1/auth/token/lookup-self"
            return self._adapter.get(path, wrap_ttl=wrap_ttl)

    def revoke_token(self, token, orphan=False, accessor=False):
        """POST /auth/token/revoke

        POST /auth/token/revoke-orphan

        POST /auth/token/revoke-accessor

        :param token:
        :type token:
        :param orphan:
        :type orphan:
        :param accessor:
        :type accessor:
        :return:
        :rtype:
        """
        if accessor and orphan:
            msg = "revoke_token does not support 'orphan' and 'accessor' flags together"
            raise exceptions.InvalidRequest(msg)
        elif accessor:
            params = {"accessor": token}
            self._adapter.post("/v1/auth/token/revoke-accessor", json=params)
        elif orphan:
            params = {"token": token}
            self._adapter.post("/v1/auth/token/revoke-orphan", json=params)
        else:
            params = {"token": token}
            self._adapter.post("/v1/auth/token/revoke", json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
    )
    def revoke_token_prefix(self, prefix):
        """POST /auth/token/revoke-prefix/<prefix>

        :param prefix:
        :type prefix:
        :return:
        :rtype:
        """
        self._adapter.post("/v1/auth/token/revoke-prefix/{0}".format(prefix))

    def renew_token(self, token=None, increment=None, wrap_ttl=None):
        """POST /auth/token/renew

        POST /auth/token/renew-self

        :param token:
        :type token:
        :param increment:
        :type increment:
        :param wrap_ttl:
        :type wrap_ttl:
        :return:
        :rtype:

        For calls expecting to hit the renew-self endpoint please use the "renew_self_token" method instead
        """
        params = {
            "increment": increment,
        }

        if token is not None:
            params["token"] = token
            return self._adapter.post(
                "/v1/auth/token/renew", json=params, wrap_ttl=wrap_ttl
            )
        else:
            generate_property_deprecation_message(
                "1.0.0",
                "renew_token() without token param",
                "renew_self_token() without token param",
                "renew_self_token",
            )
            return self.renew_self_token(increment=increment, wrap_ttl=wrap_ttl)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.renew_self,
    )
    def renew_self_token(self, increment=None, wrap_ttl=None):
        """
        POST /auth/token/renew-self

        :param increment:
        :type increment:
        :param wrap_ttl:
        :type wrap_ttl:
        :return:
        :rtype:
        """
        params = {
            "increment": increment,
        }

        return self._adapter.post(
            "/v1/auth/token/renew-self", json=params, wrap_ttl=wrap_ttl
        )

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.create_or_update_role,
    )
    def create_token_role(
        self,
        role,
        allowed_policies=None,
        disallowed_policies=None,
        orphan=None,
        period=None,
        renewable=None,
        path_suffix=None,
        explicit_max_ttl=None,
    ):
        """POST /auth/token/roles/<role>

        :param role:
        :type role:
        :param allowed_policies:
        :type allowed_policies:
        :param disallowed_policies:
        :type disallowed_policies:
        :param orphan:
        :type orphan:
        :param period:
        :type period:
        :param renewable:
        :type renewable:
        :param path_suffix:
        :type path_suffix:
        :param explicit_max_ttl:
        :type explicit_max_ttl:
        :return:
        :rtype:
        """
        params = {
            "allowed_policies": allowed_policies,
            "disallowed_policies": disallowed_policies,
            "orphan": orphan,
            "period": period,
            "renewable": renewable,
            "path_suffix": path_suffix,
            "explicit_max_ttl": explicit_max_ttl,
        }
        return self._adapter.post("/v1/auth/token/roles/{0}".format(role), json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.read_role,
    )
    def token_role(self, role):
        """Returns the named token role.

        :param role:
        :type role:
        :return:
        :rtype:
        """
        return self.read("auth/token/roles/{0}".format(role))

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.delete_role,
    )
    def delete_token_role(self, role):
        """Deletes the named token role.

        :param role:
        :type role:
        :return:
        :rtype:
        """
        return self.delete("auth/token/roles/{0}".format(role))

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Token.list_roles,
    )
    def list_token_roles(self):
        """GET /auth/token/roles?list=true

        :return:
        :rtype:
        """
        return self.list("auth/token/roles")

    def logout(self, revoke_token=False):
        """Clears the token used for authentication, optionally revoking it before doing so.

        :param revoke_token:
        :type revoke_token:
        :return:
        :rtype:
        """
        if revoke_token:
            self.revoke_self_token()

        self.token = None

    def is_authenticated(self):
        """Helper method which returns the authentication status of the client

        :return:
        :rtype:
        """
        if not self.token:
            return False

        try:
            self.lookup_token()
            return True
        except exceptions.Forbidden:
            return False
        except exceptions.InvalidPath:
            return False
        except exceptions.InvalidRequest:
            return False

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def auth_app_id(self, app_id, user_id, mount_point="app-id", use_token=True):
        """POST /auth/<mount point>/login

        :param app_id:
        :type app_id:
        :param user_id:
        :type user_id:
        :param mount_point:
        :type mount_point:
        :param use_token:
        :type use_token:
        :return:
        :rtype:
        """
        params = {
            "app_id": app_id,
            "user_id": user_id,
        }

        return self.login(
            "/v1/auth/{0}/login".format(mount_point), json=params, use_token=use_token
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.13.0",
        new_method=api.auth_methods.Cert.login,
    )
    def auth_tls(self, mount_point="cert", use_token=True):
        """POST /auth/<mount point>/login

        :param mount_point:
        :type mount_point:
        :param use_token:
        :type use_token:
        :return:
        :rtype:
        """
        return self.auth.cert.login(mount_point=mount_point, use_token=use_token)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.login,
    )
    def auth_userpass(
        self, username, password, mount_point="userpass", use_token=True, **kwargs
    ):
        """POST /auth/<mount point>/login/<username>

        :param username:
        :type username:
        :param password:
        :type password:
        :param mount_point:
        :type mount_point:
        :param use_token:
        :type use_token:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        params = {
            "password": password,
        }

        params.update(kwargs)

        return self.login(
            "/v1/auth/{0}/login/{1}".format(mount_point, username),
            json=params,
            use_token=use_token,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.iam_login,
    )
    def auth_aws_iam(
        self,
        access_key,
        secret_key,
        session_token=None,
        header_value=None,
        mount_point="aws",
        role="",
        use_token=True,
        region="us-east-1",
    ):
        """POST /auth/<mount point>/login

        :param access_key: AWS IAM access key ID
        :type access_key: str
        :param secret_key: AWS IAM secret access key
        :type secret_key: str
        :param session_token: Optional AWS IAM session token retrieved via a GetSessionToken AWS API request.
            see: https://docs.aws.amazon.com/STS/latest/APIReference/API_GetSessionToken.html
        :type session_token: str
        :param header_value: Vault allows you to require an additional header, X-Vault-AWS-IAM-Server-ID, to be present
            to mitigate against different types of replay attacks. Depending on the configuration of the AWS auth
            backend, providing a argument to this optional parameter may be required.
        :type header_value: str
        :param mount_point: The "path" the AWS auth backend was mounted on. Vault currently defaults to "aws". "aws-ec2"
            is the default argument for backwards comparability within this module.
        :type mount_point: str
        :param role: Name of the role against which the login is being attempted. If role is not specified, then the
            login endpoint looks for a role bearing the name of the AMI ID of the EC2 instance that is trying to login
            if using the ec2 auth method, or the "friendly name" (i.e., role name or username) of the IAM principal
            authenticated. If a matching role is not found, login fails.
        :type role: str
        :param use_token: If True, uses the token in the response received from the auth request to set the "token"
            attribute on the current Client class instance.
        :type use_token: bool.
        :return: The response from the AWS IAM login request attempt.
        :rtype: requests.Response
        """
        request = aws_utils.generate_sigv4_auth_request(header_value=header_value)

        auth = aws_utils.SigV4Auth(access_key, secret_key, session_token, region)
        auth.add_auth(request)

        # https://github.com/hashicorp/vault/blob/master/builtin/credential/aws/cli.go
        headers = json.dumps({k: [request.headers[k]] for k in request.headers})
        params = {
            "iam_http_request_method": request.method,
            "iam_request_url": b64encode(request.url.encode("utf-8")).decode("utf-8"),
            "iam_request_headers": b64encode(headers.encode("utf-8")).decode("utf-8"),
            "iam_request_body": b64encode(request.body.encode("utf-8")).decode("utf-8"),
            "role": role,
        }

        return self.login(
            "/v1/auth/{0}/login".format(mount_point), json=params, use_token=use_token
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.ec2_login,
    )
    def auth_ec2(
        self, pkcs7, nonce=None, role=None, use_token=True, mount_point="aws-ec2"
    ):
        """POST /auth/<mount point>/login

        :param pkcs7: PKCS#7 version of an AWS Instance Identity Document from the EC2 Metadata Service.
        :type pkcs7: str.
        :param nonce: Optional nonce returned as part of the original authentication request. Not required if the backend
            has "allow_instance_migration" or "disallow_reauthentication" options turned on.
        :type nonce: str.
        :param role: Identifier for the AWS auth backend role being requested.
        :type role: str.
        :param use_token: If True, uses the token in the response received from the auth request to set the "token"
            attribute on the current Client class instance.
        :type use_token: bool.
        :param mount_point: The "path" the AWS auth backend was mounted on. Vault currently defaults to "aws". "aws-ec2"
            is the default argument for backwards comparability within this module.
        :type mount_point: str.
        :return: parsed JSON response from the auth POST request
        :rtype: dict.

        """
        params = {"pkcs7": pkcs7}
        if nonce:
            params["nonce"] = nonce
        if role:
            params["role"] = role

        return self.login(
            "/v1/auth/{0}/login".format(mount_point), json=params, use_token=use_token
        )

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.create_or_update_user,
    )
    def create_userpass(
        self, username, password, policies, mount_point="userpass", **kwargs
    ):
        """POST /auth/<mount point>/users/<username>

        :param username:
        :type username:
        :param password:
        :type password:
        :param policies:
        :type policies:
        :param mount_point:
        :type mount_point:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        # Users can have more than 1 policy. It is easier for the user to pass in the
        # policies as a list so if they do, we need to convert to a , delimited string.
        if isinstance(policies, (list, set, tuple)):
            policies = ",".join(policies)

        params = {"password": password, "policies": policies}
        params.update(kwargs)

        return self._adapter.post(
            "/v1/auth/{}/users/{}".format(mount_point, username), json=params
        )

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.list_user,
    )
    def list_userpass(self, mount_point="userpass"):
        """GET /auth/<mount point>/users?list=true

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        try:
            return self._adapter.get(
                "/v1/auth/{}/users".format(mount_point), params={"list": True}
            )
        except exceptions.InvalidPath:
            return None

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.read_user,
    )
    def read_userpass(self, username, mount_point="userpass"):
        """GET /auth/<mount point>/users/<username>

        :param username:
        :type username:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.get("/v1/auth/{}/users/{}".format(mount_point, username))

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.create_or_update_user,
    )
    def update_userpass_policies(self, username, policies, mount_point="userpass"):
        """POST /auth/<mount point>/users/<username>/policies

        :param username:
        :type username:
        :param policies:
        :type policies:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        # userpass can have more than 1 policy. It is easier for the user to pass in the
        # policies as a list so if they do, we need to convert to a , delimited string.
        if isinstance(policies, (list, set, tuple)):
            policies = ",".join(policies)

        params = {"policies": policies}

        return self._adapter.post(
            "/v1/auth/{}/users/{}/policies".format(mount_point, username), json=params
        )

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.update_password_on_user,
    )
    def update_userpass_password(self, username, password, mount_point="userpass"):
        """POST /auth/<mount point>/users/<username>/password

        :param username:
        :type username:
        :param password:
        :type password:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        params = {"password": password}
        return self._adapter.post(
            "/v1/auth/{}/users/{}/password".format(mount_point, username), json=params
        )

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Userpass.delete_user,
    )
    def delete_userpass(self, username, mount_point="userpass"):
        """DELETE /auth/<mount point>/users/<username>

        :param username:
        :type username:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.delete(
            "/v1/auth/{}/users/{}".format(mount_point, username)
        )

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def create_app_id(
        self, app_id, policies, display_name=None, mount_point="app-id", **kwargs
    ):
        """POST /auth/<mount point>/map/app-id/<app_id>

        :param app_id:
        :type app_id:
        :param policies:
        :type policies:
        :param display_name:
        :type display_name:
        :param mount_point:
        :type mount_point:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        # app-id can have more than 1 policy. It is easier for the user to pass in the
        # policies as a list so if they do, we need to convert to a , delimited string.
        if isinstance(policies, (list, set, tuple)):
            policies = ",".join(policies)

        params = {"value": policies}

        # Only use the display_name if it has a value. Made it a named param for user
        # convienence instead of leaving it as part of the kwargs
        if display_name:
            params["display_name"] = display_name

        params.update(kwargs)

        return self._adapter.post(
            "/v1/auth/{}/map/app-id/{}".format(mount_point, app_id), json=params
        )

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def get_app_id(self, app_id, mount_point="app-id", wrap_ttl=None):
        """GET /auth/<mount_point>/map/app-id/<app_id>

        :param app_id:
        :type app_id:
        :param mount_point:
        :type mount_point:
        :param wrap_ttl:
        :type wrap_ttl:
        :return:
        :rtype:
        """
        path = "/v1/auth/{0}/map/app-id/{1}".format(mount_point, app_id)
        return self._adapter.get(path, wrap_ttl=wrap_ttl)

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def delete_app_id(self, app_id, mount_point="app-id"):
        """DELETE /auth/<mount_point>/map/app-id/<app_id>

        :param app_id:
        :type app_id:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.delete(
            "/v1/auth/{0}/map/app-id/{1}".format(mount_point, app_id)
        )

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def create_user_id(
        self, user_id, app_id, cidr_block=None, mount_point="app-id", **kwargs
    ):
        """POST /auth/<mount point>/map/user-id/<user_id>

        :param user_id:
        :type user_id:
        :param app_id:
        :type app_id:
        :param cidr_block:
        :type cidr_block:
        :param mount_point:
        :type mount_point:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        # user-id can be associated to more than 1 app-id (aka policy). It is easier for the user to
        # pass in the policies as a list so if they do, we need to convert to a , delimited string.
        if isinstance(app_id, (list, set, tuple)):
            app_id = ",".join(app_id)

        params = {"value": app_id}

        # Only use the cidr_block if it has a value. Made it a named param for user
        # convienence instead of leaving it as part of the kwargs
        if cidr_block:
            params["cidr_block"] = cidr_block

        params.update(kwargs)

        return self._adapter.post(
            "/v1/auth/{}/map/user-id/{}".format(mount_point, user_id), json=params
        )

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def get_user_id(self, user_id, mount_point="app-id", wrap_ttl=None):
        """GET /auth/<mount_point>/map/user-id/<user_id>

        :param user_id:
        :type user_id:
        :param mount_point:
        :type mount_point:
        :param wrap_ttl:
        :type wrap_ttl:
        :return:
        :rtype:
        """
        path = "/v1/auth/{0}/map/user-id/{1}".format(mount_point, user_id)
        return self._adapter.get(path, wrap_ttl=wrap_ttl)

    @utils.deprecated_method(to_be_removed_in_version="1.0.0")
    def delete_user_id(self, user_id, mount_point="app-id"):
        """DELETE /auth/<mount_point>/map/user-id/<user_id>

        :param user_id:
        :type user_id:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.delete(
            "/v1/auth/{0}/map/user-id/{1}".format(mount_point, user_id)
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.configure,
    )
    def create_vault_ec2_client_configuration(
        self, access_key, secret_key, endpoint=None, mount_point="aws-ec2"
    ):
        """POST /auth/<mount_point>/config/client

        Configure the credentials required to perform API calls to AWS as well as custom endpoints to talk to AWS APIs.
        The instance identity document fetched from the PKCS#7 signature will provide the EC2 instance ID. The
        credentials configured using this endpoint will be used to query the status of the instances via
        DescribeInstances API. If static credentials are not provided using this endpoint, then the credentials will be
        retrieved from the environment variables AWS_ACCESS_KEY, AWS_SECRET_KEY and AWS_REGION respectively. If the
        credentials are still not found and if the method is configured on an EC2 instance with metadata querying
        capabilities, the credentials are fetched automatically

        :param access_key: AWS Access key with permissions to query AWS APIs. The permissions required depend on the
            specific configurations. If using the iam auth method without inferencing, then no credentials are
            necessary. If using the ec2 auth method or using the iam auth method with inferencing, then these
            credentials need access to ec2:DescribeInstances. If additionally a bound_iam_role is specified, then these
            credentials also need access to iam:GetInstanceProfile. If, however, an alternate sts configuration is set
            for the target account, then the credentials must be permissioned to call sts:AssumeRole on the configured
            role, and that role must have the permissions described here.
        :type access_key: str|unicode
        :param secret_key: AWS Secret key with permissions to query AWS APIs.
        :type secret_key: str|unicode
        :param endpoint: URL to override the default generated endpoint for making AWS EC2 API calls.
        :type endpoint: str|unicode
        :param mount_point: The "path" the AWS auth backend was mounted on. Vault currently defaults to "aws". "aws-ec2"
            is the default argument for backwards comparability within this module.
        :type mount_point: str|unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {"access_key": access_key, "secret_key": secret_key}
        if endpoint is not None:
            params["endpoint"] = endpoint

        return self._adapter.post(
            "/v1/auth/{0}/config/client".format(mount_point), json=params
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.read_config,
    )
    def get_vault_ec2_client_configuration(self, mount_point="aws-ec2"):
        """GET /auth/<mount_point>/config/client

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.get("/v1/auth/{0}/config/client".format(mount_point))

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.delete_config,
    )
    def delete_vault_ec2_client_configuration(self, mount_point="aws-ec2"):
        """DELETE /auth/<mount_point>/config/client

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.delete("/v1/auth/{0}/config/client".format(mount_point))

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.create_certificate_configuration,
    )
    def create_vault_ec2_certificate_configuration(
        self, cert_name, aws_public_cert, mount_point="aws-ec2"
    ):
        """POST /auth/<mount_point>/config/certificate/<cert_name>

        :param cert_name:
        :type cert_name:
        :param aws_public_cert:
        :type aws_public_cert:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        params = {"cert_name": cert_name, "aws_public_cert": aws_public_cert}
        return self._adapter.post(
            "/v1/auth/{0}/config/certificate/{1}".format(mount_point, cert_name),
            json=params,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.read_certificate_configuration,
    )
    def get_vault_ec2_certificate_configuration(self, cert_name, mount_point="aws-ec2"):
        """GET /auth/<mount_point>/config/certificate/<cert_name>

        :param cert_name:
        :type cert_name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.get(
            "/v1/auth/{0}/config/certificate/{1}".format(mount_point, cert_name)
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.list_certificate_configurations,
    )
    def list_vault_ec2_certificate_configurations(self, mount_point="aws-ec2"):
        """GET /auth/<mount_point>/config/certificates?list=true

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        params = {"list": True}
        return self._adapter.get(
            "/v1/auth/{0}/config/certificates".format(mount_point), params=params
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.create_role,
    )
    def create_ec2_role(
        self,
        role,
        bound_ami_id=None,
        bound_account_id=None,
        bound_iam_role_arn=None,
        bound_iam_instance_profile_arn=None,
        bound_ec2_instance_id=None,
        bound_region=None,
        bound_vpc_id=None,
        bound_subnet_id=None,
        role_tag=None,
        ttl=None,
        max_ttl=None,
        period=None,
        policies=None,
        allow_instance_migration=False,
        disallow_reauthentication=False,
        resolve_aws_unique_ids=None,
        mount_point="aws-ec2",
    ):
        """POST /auth/<mount_point>/role/<role>

        :param role:
        :type role:
        :param bound_ami_id:
        :type bound_ami_id:
        :param bound_account_id:
        :type bound_account_id:
        :param bound_iam_role_arn:
        :type bound_iam_role_arn:
        :param bound_iam_instance_profile_arn:
        :type bound_iam_instance_profile_arn:
        :param bound_ec2_instance_id:
        :type bound_ec2_instance_id:
        :param bound_region:
        :type bound_region:
        :param bound_vpc_id:
        :type bound_vpc_id:
        :param bound_subnet_id:
        :type bound_subnet_id:
        :param role_tag:
        :type role_tag:
        :param ttl:
        :type ttl:
        :param max_ttl:
        :type max_ttl:
        :param period:
        :type period:
        :param policies:
        :type policies:
        :param allow_instance_migration:
        :type allow_instance_migration:
        :param disallow_reauthentication:
        :type disallow_reauthentication:
        :param resolve_aws_unique_ids:
        :type resolve_aws_unique_ids:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        params = {
            "role": role,
            "auth_type": "ec2",
            "disallow_reauthentication": disallow_reauthentication,
            "allow_instance_migration": allow_instance_migration,
        }

        if bound_ami_id is not None:
            params["bound_ami_id"] = bound_ami_id
        if bound_account_id is not None:
            params["bound_account_id"] = bound_account_id
        if bound_iam_role_arn is not None:
            params["bound_iam_role_arn"] = bound_iam_role_arn
        if bound_ec2_instance_id is not None:
            params["bound_iam_instance_profile_arn"] = bound_ec2_instance_id
        if bound_iam_instance_profile_arn is not None:
            params["bound_iam_instance_profile_arn"] = bound_iam_instance_profile_arn
        if bound_region is not None:
            params["bound_region"] = bound_region
        if bound_vpc_id is not None:
            params["bound_vpc_id"] = bound_vpc_id
        if bound_subnet_id is not None:
            params["bound_subnet_id"] = bound_subnet_id
        if role_tag is not None:
            params["role_tag"] = role_tag
        if ttl is not None:
            params["ttl"] = ttl
        else:
            params["ttl"] = 0
        if max_ttl is not None:
            params["max_ttl"] = max_ttl
        else:
            params["max_ttl"] = 0
        if period is not None:
            params["period"] = period
        else:
            params["period"] = 0
        if policies is not None:
            params["policies"] = policies
        if resolve_aws_unique_ids is not None:
            params["resolve_aws_unique_ids"] = resolve_aws_unique_ids

        return self._adapter.post(
            "/v1/auth/{0}/role/{1}".format(mount_point, role), json=params
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.read_role,
    )
    def get_ec2_role(self, role, mount_point="aws-ec2"):
        """GET /auth/<mount_point>/role/<role>

        :param role:
        :type role:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.get("/v1/auth/{0}/role/{1}".format(mount_point, role))

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.delete_role,
    )
    def delete_ec2_role(self, role, mount_point="aws-ec2"):
        """DELETE /auth/<mount_point>/role/<role>

        :param role:
        :type role:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.delete("/v1/auth/{0}/role/{1}".format(mount_point, role))

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.list_roles,
    )
    def list_ec2_roles(self, mount_point="aws-ec2"):
        """GET /auth/<mount_point>/roles?list=true

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        try:
            return self._adapter.get(
                "/v1/auth/{0}/roles".format(mount_point), params={"list": True}
            )
        except exceptions.InvalidPath:
            return None

    @utils.deprecated_method(
        to_be_removed_in_version="0.11.2",
        new_method=api.auth_methods.Aws.create_role_tags,
    )
    def create_ec2_role_tag(
        self,
        role,
        policies=None,
        max_ttl=None,
        instance_id=None,
        disallow_reauthentication=False,
        allow_instance_migration=False,
        mount_point="aws-ec2",
    ):
        """POST /auth/<mount_point>/role/<role>/tag

        :param role: Name of the role.
        :type role: str
        :param policies: Policies to be associated with the tag. If set, must be a subset of the role's policies. If
            set, but set to an empty value, only the 'default' policy will be given to issued tokens.
        :type policies: list
        :param max_ttl: The maximum allowed lifetime of tokens issued using this role.
        :type max_ttl: str
        :param instance_id: Instance ID for which this tag is intended for. If set, the created tag can only be used by
            the instance with the given ID.
        :type instance_id: str
        :param disallow_reauthentication: If set, only allows a single token to be granted per instance ID. This can be
            cleared with the auth/aws/identity-whitelist endpoint. Defaults to 'false'. Mutually exclusive with
            allow_instance_migration.
        :type disallow_reauthentication: bool
        :param allow_instance_migration: If set, allows migration of the underlying instance where the client resides.
            This keys off of pendingTime in the metadata document, so essentially, this disables the client nonce check
            whenever the instance is migrated to a new host and pendingTime is newer than the previously-remembered
            time. Use with caution. Defaults to 'false'. Mutually exclusive with disallow_reauthentication.
        :type allow_instance_migration: bool
        :param mount_point: The path the associated AWS auth method is configured under.
        :type mount_point: str
        :return: The create role tag response.
        :rtype: dict
        """
        params = {
            "role": role,
            "disallow_reauthentication": disallow_reauthentication,
            "allow_instance_migration": allow_instance_migration,
        }

        if max_ttl is not None:
            params["max_ttl"] = max_ttl
        if policies is not None:
            params["policies"] = policies
        if instance_id is not None:
            params["instance_id"] = instance_id
        return self._adapter.post(
            "/v1/auth/{0}/role/{1}/tag".format(mount_point, role), json=params
        )

    def auth_cubbyhole(self, token):
        """Perform a login request with a wrapped token.

        Stores the unwrapped token in the resulting Vault response for use by the :py:meth:`hvac.adapters.Adapter`
            instance under the _adapater Client attribute.

        :param token: Wrapped token
        :type token: str | unicode
        :return: The (JSON decoded) response of the auth request
        :rtype: dict
        """
        self.token = token
        return self.login("/v1/sys/wrapping/unwrap")

    def login(self, url, use_token=True, **kwargs):
        """Perform a login request.

        Associated request is typically to a path prefixed with "/v1/auth") and optionally stores the client token sent
            in the resulting Vault response for use by the :py:meth:`hvac.adapters.Adapter` instance under the _adapater
            Client attribute.

        :param url: Path to send the authentication request to.
        :type url: str | unicode
        :param use_token: if True, uses the token in the response received from the auth request to set the "token"
            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapater Client attribute.
        :type use_token: bool
        :param kwargs: Additional keyword arguments to include in the params sent with the request.
        :type kwargs: dict
        :return: The response of the auth request.
        :rtype: requests.Response
        """
        return self._adapter.login(url=url, use_token=use_token, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.create_or_update_approle,
    )
    def create_role(self, role_name, mount_point="approle", **kwargs):
        """POST /auth/<mount_point>/role/<role name>

        :param role_name:
        :type role_name:
        :param mount_point:
        :type mount_point:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        return self._adapter.post(
            "/v1/auth/{0}/role/{1}".format(mount_point, role_name), json=kwargs
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.delete_role,
    )
    def delete_role(self, role_name, mount_point="approle"):
        """DELETE /auth/<mount_point>/role/<role name>

        :param role_name:
        :type role_name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """

        return self._adapter.delete(
            "/v1/auth/{0}/role/{1}".format(mount_point, role_name)
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.list_roles,
    )
    def list_roles(self, mount_point="approle"):
        """GET /auth/<mount_point>/role

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """

        return self._adapter.get("/v1/auth/{0}/role?list=true".format(mount_point))

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.read_role_id,
    )
    def get_role_id(self, role_name, mount_point="approle"):
        """GET /auth/<mount_point>/role/<role name>/role-id

        :param role_name:
        :type role_name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """

        url = "/v1/auth/{0}/role/{1}/role-id".format(mount_point, role_name)
        return self._adapter.get(url)["data"]["role_id"]

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.update_role_id,
    )
    def set_role_id(self, role_name, role_id, mount_point="approle"):
        """POST /auth/<mount_point>/role/<role name>/role-id

        :param role_name:
        :type role_name:
        :param role_id:
        :type role_id:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """

        url = "/v1/auth/{0}/role/{1}/role-id".format(mount_point, role_name)
        params = {"role_id": role_id}
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.read_role,
    )
    def get_role(self, role_name, mount_point="approle"):
        """GET /auth/<mount_point>/role/<role name>

        :param role_name:
        :type role_name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        return self._adapter.get("/v1/auth/{0}/role/{1}".format(mount_point, role_name))

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.generate_secret_id,
    )
    def create_role_secret_id(
        self,
        role_name,
        meta=None,
        cidr_list=None,
        token_bound_cidrs=None,
        wrap_ttl=None,
        mount_point="approle",
    ):
        """POST /auth/<mount_point>/role/<role name>/secret-id

        :param role_name:
        :type role_name:
        :param meta:
        :type meta:
        :param cidr_list:
        :type cidr_list:
        :param token_bound_cidrs:
        :type token_bound_cidrs:
        :param wrap_ttl:
        :type wrap_ttl:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """

        url = "/v1/auth/{0}/role/{1}/secret-id".format(mount_point, role_name)
        params = {}
        if meta is not None:
            params["metadata"] = json.dumps(meta)
        if cidr_list is not None:
            params["cidr_list"] = cidr_list
        if token_bound_cidrs is not None:
            params["token_bound_cidrs"] = token_bound_cidrs
        return self._adapter.post(url, json=params, wrap_ttl=wrap_ttl)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.read_secret_id,
    )
    def get_role_secret_id(self, role_name, secret_id, mount_point="approle"):
        """POST /auth/<mount_point>/role/<role name>/secret-id/lookup

        :param role_name:
        :type role_name:
        :param secret_id:
        :type secret_id:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/auth/{0}/role/{1}/secret-id/lookup".format(mount_point, role_name)
        params = {"secret_id": secret_id}
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.list_secret_id_accessors,
    )
    def list_role_secrets(self, role_name, mount_point="approle"):
        """LIST /auth/<mount_point>/role/<role name>/secret-id

        :param role_name: Name of the AppRole.
        :type role_name: str|unicode
        :param mount_point: The "path" the AppRole auth backend was mounted on. Vault currently defaults to "approle".
        :type mount_point: str|unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        url = "/v1/auth/{mount_point}/role/{name}/secret-id".format(
            mount_point=mount_point, name=role_name
        )
        return self._adapter.list(url)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.read_secret_id_accessor,
    )
    def get_role_secret_id_accessor(
        self, role_name, secret_id_accessor, mount_point="approle"
    ):
        """POST /auth/<mount_point>/role/<role name>/secret-id-accessor/lookup

        :param role_name:
        :type role_name:
        :param secret_id_accessor:
        :type secret_id_accessor:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/auth/{0}/role/{1}/secret-id-accessor/lookup".format(
            mount_point, role_name
        )
        params = {"secret_id_accessor": secret_id_accessor}
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.destroy_secret_id,
    )
    def delete_role_secret_id(self, role_name, secret_id, mount_point="approle"):
        """POST /auth/<mount_point>/role/<role name>/secret-id/destroy

        :param role_name:
        :type role_name:
        :param secret_id:
        :type secret_id:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/auth/{0}/role/{1}/secret-id/destroy".format(mount_point, role_name)
        params = {"secret_id": secret_id}
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.destroy_secret_id_accessor,
    )
    def delete_role_secret_id_accessor(
        self, role_name, secret_id_accessor, mount_point="approle"
    ):
        """POST /auth/<mount_point>/role/<role name>/secret-id-accessor/destroy

        :param role_name:
        :type role_name:
        :param secret_id_accessor:
        :type secret_id_accessor:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/auth/{0}/role/{1}/secret-id-accessor/destroy".format(
            mount_point, role_name
        )
        params = {"secret_id_accessor": secret_id_accessor}
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.create_custom_secret_id,
    )
    def create_role_custom_secret_id(
        self, role_name, secret_id, meta=None, mount_point="approle"
    ):
        """POST /auth/<mount_point>/role/<role name>/custom-secret-id

        :param role_name:
        :type role_name:
        :param secret_id:
        :type secret_id:
        :param meta:
        :type meta:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/auth/{0}/role/{1}/custom-secret-id".format(mount_point, role_name)
        params = {"secret_id": secret_id}
        if meta is not None:
            params["meta"] = meta
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.12.0",
        new_method=api.auth_methods.AppRole.login,
    )
    def auth_approle(
        self, role_id, secret_id=None, mount_point="approle", use_token=True
    ):
        """POST /auth/<mount_point>/login

        :param role_id:
        :type role_id:
        :param secret_id:
        :type secret_id:
        :param mount_point:
        :type mount_point:
        :param use_token:
        :type use_token:
        :return:
        :rtype:
        """
        params = {"role_id": role_id}
        if secret_id is not None:
            params["secret_id"] = secret_id

        return self.login(
            "/v1/auth/{0}/login".format(mount_point), json=params, use_token=use_token
        )

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.configure,
    )
    def create_kubernetes_configuration(
        self,
        kubernetes_host,
        kubernetes_ca_cert=None,
        token_reviewer_jwt=None,
        pem_keys=None,
        mount_point="kubernetes",
    ):
        """POST /auth/<mount_point>/config

        :param kubernetes_host: A host:port pair, or a URL to the base of the Kubernetes API server.
        :type kubernetes_host: str.
        :param kubernetes_ca_cert: PEM encoded CA cert for use by the TLS client used to talk with the Kubernetes API.
        :type kubernetes_ca_cert: str.
        :param token_reviewer_jwt: A service account JWT used to access the TokenReview API to validate other
            JWTs during login. If not set the JWT used for login will be used to access the API.
        :type token_reviewer_jwt: str.
        :param pem_keys: Optional list of PEM-formated public keys or certificates used to verify the signatures of
            Kubernetes service account JWTs. If a certificate is given, its public key will be extracted. Not every
            installation of Kubernetes exposes these keys.
        :type pem_keys: list.
        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :return: Will be an empty body with a 204 status code upon success
        :rtype: requests.Response.
        """
        params = {
            "kubernetes_host": kubernetes_host,
            "kubernetes_ca_cert": kubernetes_ca_cert,
        }

        if token_reviewer_jwt is not None:
            params["token_reviewer_jwt"] = token_reviewer_jwt
        if pem_keys is not None:
            params["pem_keys"] = pem_keys

        url = "v1/auth/{0}/config".format(mount_point)
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.read_config,
    )
    def get_kubernetes_configuration(self, mount_point="kubernetes"):
        """GET /auth/<mount_point>/config

        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :return: Parsed JSON response from the config GET request
        :rtype: dict.
        """

        url = "/v1/auth/{0}/config".format(mount_point)
        return self._adapter.get(url)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.create_role,
    )
    def create_kubernetes_role(
        self,
        name,
        bound_service_account_names,
        bound_service_account_namespaces,
        ttl="",
        max_ttl="",
        period="",
        policies=None,
        token_type="",
        mount_point="kubernetes",
    ):
        """POST /auth/<mount_point>/role/:name

        :param name: Name of the role.
        :type name: str.
        :param bound_service_account_names: List of service account names able to access this role. If set to "*" all
            names are allowed, both this and bound_service_account_namespaces can not be "*".
        :type bound_service_account_names: list.
        :param bound_service_account_namespaces: List of namespaces allowed to access this role. If set to "*" all
            namespaces are allowed, both this and bound_service_account_names can not be set to "*".
        :type bound_service_account_namespaces: list.
        :param ttl: The TTL period of tokens issued using this role in seconds.
        :type ttl: str.
        :param max_ttl: The maximum allowed lifetime of tokens issued in seconds using this role.
        :type max_ttl: str.
        :param period: If set, indicates that the token generated using this role should never expire.
            The token should be renewed within the duration specified by this value. At each renewal, the token's TTL will
            be set to the value of this parameter.
        :type period: str.
        :param policies: Policies to be set on tokens issued using this role
        :type policies: list.
        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :type token_type: str.
        :return: Will be an empty body with a 204 status code upon success
        :rtype: requests.Response.
        """
        if (
            bound_service_account_names == "*"
            and bound_service_account_namespaces == "*"
        ):
            error_message = 'bound_service_account_names and bound_service_account_namespaces can not both be set to "*"'
            raise exceptions.ParamValidationError(error_message)

        params = {
            "bound_service_account_names": bound_service_account_names,
            "bound_service_account_namespaces": bound_service_account_namespaces,
            "ttl": ttl,
            "max_ttl": max_ttl,
            "period": period,
            "policies": policies,
        }
        if token_type:
            params["token_type"] = token_type

        url = "v1/auth/{0}/role/{1}".format(mount_point, name)
        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.read_role,
    )
    def get_kubernetes_role(self, name, mount_point="kubernetes"):
        """GET /auth/<mount_point>/role/:name

        :param name: Name of the role.
        :type name: str.
        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :return: Parsed JSON response from the read role GET request
        :rtype: dict.
        """

        url = "v1/auth/{0}/role/{1}".format(mount_point, name)
        return self._adapter.get(url)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.list_roles,
    )
    def list_kubernetes_roles(self, mount_point="kubernetes"):
        """GET /auth/<mount_point>/role?list=true

        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :return: Parsed JSON response from the list roles GET request.
        :rtype: dict.
        """

        url = "v1/auth/{0}/role?list=true".format(mount_point)
        return self._adapter.get(url)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.delete_role,
    )
    def delete_kubernetes_role(self, role, mount_point="kubernetes"):
        """DELETE /auth/<mount_point>/role/:role

        :type role: Name of the role.
        :param role: str.
        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :return: Will be an empty body with a 204 status code upon success.
        :rtype: requests.Response.
        """

        url = "v1/auth/{0}/role/{1}".format(mount_point, role)
        return self._adapter.delete(url)

    @utils.deprecated_method(
        to_be_removed_in_version="1.0.0",
        new_method=api.auth_methods.Kubernetes.login,
    )
    def auth_kubernetes(self, role, jwt, use_token=True, mount_point="kubernetes"):
        """POST /auth/<mount_point>/login

        :param role: Name of the role against which the login is being attempted.
        :type role: str.
        :param jwt: Signed JSON Web Token (JWT) for authenticating a service account.
        :type jwt: str.
        :param use_token: if True, uses the token in the response received from the auth request to set the "token"
            attribute on the current Client class instance.
        :type use_token: bool.
        :param mount_point: The "path" the k8s auth backend was mounted on. Vault currently defaults to "kubernetes".
        :type mount_point: str.
        :return: Parsed JSON response from the config POST request.
        :rtype: dict.
        """
        params = {"role": role, "jwt": jwt}
        url = "v1/auth/{0}/login".format(mount_point)
        return self.login(url, json=params, use_token=use_token)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.create_key,
    )
    def transit_create_key(
        self,
        name,
        convergent_encryption=None,
        derived=None,
        exportable=None,
        key_type=None,
        mount_point="transit",
    ):
        """POST /<mount_point>/keys/<name>

        :param name:
        :type name:
        :param convergent_encryption:
        :type convergent_encryption:
        :param derived:
        :type derived:
        :param exportable:
        :type exportable:
        :param key_type:
        :type key_type:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/keys/{1}".format(mount_point, name)
        params = {}
        if convergent_encryption is not None:
            params["convergent_encryption"] = convergent_encryption
        if derived is not None:
            params["derived"] = derived
        if exportable is not None:
            params["exportable"] = exportable
        if key_type is not None:
            params["type"] = key_type

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.read_key,
    )
    def transit_read_key(self, name, mount_point="transit"):
        """GET /<mount_point>/keys/<name>

        :param name:
        :type name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/keys/{1}".format(mount_point, name)
        return self._adapter.get(url)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.list_keys,
    )
    def transit_list_keys(self, mount_point="transit"):
        """GET /<mount_point>/keys?list=true

        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/keys?list=true".format(mount_point)
        return self._adapter.get(url)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.delete_key,
    )
    def transit_delete_key(self, name, mount_point="transit"):
        """DELETE /<mount_point>/keys/<name>

        :param name:
        :type name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/keys/{1}".format(mount_point, name)
        return self._adapter.delete(url)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.update_key_configuration,
    )
    def transit_update_key(
        self,
        name,
        min_decryption_version=None,
        min_encryption_version=None,
        deletion_allowed=None,
        mount_point="transit",
    ):
        """POST /<mount_point>/keys/<name>/config

        :param name:
        :type name:
        :param min_decryption_version:
        :type min_decryption_version:
        :param min_encryption_version:
        :type min_encryption_version:
        :param deletion_allowed:
        :type deletion_allowed:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/keys/{1}/config".format(mount_point, name)
        params = {}
        if min_decryption_version is not None:
            params["min_decryption_version"] = min_decryption_version
        if min_encryption_version is not None:
            params["min_encryption_version"] = min_encryption_version
        if deletion_allowed is not None:
            params["deletion_allowed"] = deletion_allowed

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.rotate_key,
    )
    def transit_rotate_key(self, name, mount_point="transit"):
        """POST /<mount_point>/keys/<name>/rotate

        :param name:
        :type name:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/keys/{1}/rotate".format(mount_point, name)
        return self._adapter.post(url)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.export_key,
    )
    def transit_export_key(self, name, key_type, version=None, mount_point="transit"):
        """GET /<mount_point>/export/<key_type>/<name>(/<version>)

        :param name:
        :type name:
        :param key_type:
        :type key_type:
        :param version:
        :type version:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        if version is not None:
            url = "/v1/{0}/export/{1}/{2}/{3}".format(
                mount_point, key_type, name, version
            )
        else:
            url = "/v1/{0}/export/{1}/{2}".format(mount_point, key_type, name)
        return self._adapter.get(url)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.encrypt_data,
    )
    def transit_encrypt_data(
        self,
        name,
        plaintext,
        context=None,
        key_version=None,
        nonce=None,
        batch_input=None,
        key_type=None,
        convergent_encryption=None,
        mount_point="transit",
    ):
        """POST /<mount_point>/encrypt/<name>

        :param name:
        :type name:
        :param plaintext:
        :type plaintext:
        :param context:
        :type context:
        :param key_version:
        :type key_version:
        :param nonce:
        :type nonce:
        :param batch_input:
        :type batch_input:
        :param key_type:
        :type key_type:
        :param convergent_encryption:
        :type convergent_encryption:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/encrypt/{1}".format(mount_point, name)
        params = {"plaintext": plaintext}
        if context is not None:
            params["context"] = context
        if key_version is not None:
            params["key_version"] = key_version
        if nonce is not None:
            params["nonce"] = nonce
        if batch_input is not None:
            params["batch_input"] = batch_input
        if key_type is not None:
            params["type"] = key_type
        if convergent_encryption is not None:
            params["convergent_encryption"] = convergent_encryption

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.decrypt_data,
    )
    def transit_decrypt_data(
        self,
        name,
        ciphertext,
        context=None,
        nonce=None,
        batch_input=None,
        mount_point="transit",
    ):
        """POST /<mount_point>/decrypt/<name>

        :param name:
        :type name:
        :param ciphertext:
        :type ciphertext:
        :param context:
        :type context:
        :param nonce:
        :type nonce:
        :param batch_input:
        :type batch_input:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/decrypt/{1}".format(mount_point, name)
        params = {"ciphertext": ciphertext}
        if context is not None:
            params["context"] = context
        if nonce is not None:
            params["nonce"] = nonce
        if batch_input is not None:
            params["batch_input"] = batch_input

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.rewrap_data,
    )
    def transit_rewrap_data(
        self,
        name,
        ciphertext,
        context=None,
        key_version=None,
        nonce=None,
        batch_input=None,
        mount_point="transit",
    ):
        """POST /<mount_point>/rewrap/<name>

        :param name:
        :type name:
        :param ciphertext:
        :type ciphertext:
        :param context:
        :type context:
        :param key_version:
        :type key_version:
        :param nonce:
        :type nonce:
        :param batch_input:
        :type batch_input:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/rewrap/{1}".format(mount_point, name)
        params = {"ciphertext": ciphertext}
        if context is not None:
            params["context"] = context
        if key_version is not None:
            params["key_version"] = key_version
        if nonce is not None:
            params["nonce"] = nonce
        if batch_input is not None:
            params["batch_input"] = batch_input

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.generate_data_key,
    )
    def transit_generate_data_key(
        self, name, key_type, context=None, nonce=None, bits=None, mount_point="transit"
    ):
        """POST /<mount_point>/datakey/<type>/<name>

        :param name:
        :type name:
        :param key_type:
        :type key_type:
        :param context:
        :type context:
        :param nonce:
        :type nonce:
        :param bits:
        :type bits:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        url = "/v1/{0}/datakey/{1}/{2}".format(mount_point, key_type, name)
        params = {}
        if context is not None:
            params["context"] = context
        if nonce is not None:
            params["nonce"] = nonce
        if bits is not None:
            params["bits"] = bits

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.generate_random_bytes,
    )
    def transit_generate_rand_bytes(
        self, data_bytes=None, output_format=None, mount_point="transit"
    ):
        """POST /<mount_point>/random(/<data_bytes>)

        :param data_bytes:
        :type data_bytes:
        :param output_format:
        :type output_format:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        if data_bytes is not None:
            url = "/v1/{0}/random/{1}".format(mount_point, data_bytes)
        else:
            url = "/v1/{0}/random".format(mount_point)

        params = {}
        if output_format is not None:
            params["format"] = output_format

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.hash_data,
    )
    def transit_hash_data(
        self, hash_input, algorithm=None, output_format=None, mount_point="transit"
    ):
        """POST /<mount_point>/hash(/<algorithm>)

        :param hash_input:
        :type hash_input:
        :param algorithm:
        :type algorithm:
        :param output_format:
        :type output_format:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        if algorithm is not None:
            url = "/v1/{0}/hash/{1}".format(mount_point, algorithm)
        else:
            url = "/v1/{0}/hash".format(mount_point)

        params = {"input": hash_input}
        if output_format is not None:
            params["format"] = output_format

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.generate_hmac,
    )
    def transit_generate_hmac(
        self, name, hmac_input, key_version=None, algorithm=None, mount_point="transit"
    ):
        """POST /<mount_point>/hmac/<name>(/<algorithm>)

        :param name:
        :type name:
        :param hmac_input:
        :type hmac_input:
        :param key_version:
        :type key_version:
        :param algorithm:
        :type algorithm:
        :param mount_point:
        :type mount_point:
        :return:
        :rtype:
        """
        if algorithm is not None:
            url = "/v1/{0}/hmac/{1}/{2}".format(mount_point, name, algorithm)
        else:
            url = "/v1/{0}/hmac/{1}".format(mount_point, name)
        params = {"input": hmac_input}
        if key_version is not None:
            params["key_version"] = key_version

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.sign_data,
    )
    def transit_sign_data(
        self,
        name,
        input_data,
        key_version=None,
        algorithm=None,
        context=None,
        prehashed=None,
        mount_point="transit",
        signature_algorithm="pss",
    ):
        """POST /<mount_point>/sign/<name>(/<algorithm>)

        :param name:
        :type name:
        :param input_data:
        :type input_data:
        :param key_version:
        :type key_version:
        :param algorithm:
        :type algorithm:
        :param context:
        :type context:
        :param prehashed:
        :type prehashed:
        :param mount_point:
        :type mount_point:
        :param signature_algorithm:
        :type signature_algorithm:
        :return:
        :rtype:
        """
        if algorithm is not None:
            url = "/v1/{0}/sign/{1}/{2}".format(mount_point, name, algorithm)
        else:
            url = "/v1/{0}/sign/{1}".format(mount_point, name)

        params = {"input": input_data}
        if key_version is not None:
            params["key_version"] = key_version
        if context is not None:
            params["context"] = context
        if prehashed is not None:
            params["prehashed"] = prehashed
        params["signature_algorithm"] = signature_algorithm

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.secrets_engines.Transit.verify_signed_data,
    )
    def transit_verify_signed_data(
        self,
        name,
        input_data,
        algorithm=None,
        signature=None,
        hmac=None,
        context=None,
        prehashed=None,
        mount_point="transit",
        signature_algorithm="pss",
    ):
        """POST /<mount_point>/verify/<name>(/<algorithm>)

        :param name:
        :type name:
        :param input_data:
        :type input_data:
        :param algorithm:
        :type algorithm:
        :param signature:
        :type signature:
        :param hmac:
        :type hmac:
        :param context:
        :type context:
        :param prehashed:
        :type prehashed:
        :param mount_point:
        :type mount_point:
        :param signature_algorithm:
        :type signature_algorithm:
        :return:
        :rtype:
        """
        if algorithm is not None:
            url = "/v1/{0}/verify/{1}/{2}".format(mount_point, name, algorithm)
        else:
            url = "/v1/{0}/verify/{1}".format(mount_point, name)

        params = {"input": input_data}
        if signature is not None:
            params["signature"] = signature
        if hmac is not None:
            params["hmac"] = hmac
        if context is not None:
            params["context"] = context
        if prehashed is not None:
            params["prehashed"] = prehashed
        params["signature_algorithm"] = signature_algorithm

        return self._adapter.post(url, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.unwrap,
    )
    def unwrap(self, token=None):
        return self.sys.unwrap(token=token)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.list_policies,
    )
    def list_policies(self):
        policies = self.sys.list_policies()["data"]["policies"]
        return policies

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.create_or_update_policy,
    )
    def set_policy(self, name, rules):
        """Add a new or update an existing policy.

        Once a policy is updated, it takes effect immediately to all associated users.

        Supported methods:
            PUT: /sys/policy/{name}. Produces: 204 (empty body)

        :param name: Specifies the name of the policy to create.
        :type name: str | unicode
        :param policy: Specifies the policy document.
        :type policy: str | unicode | dict
        """
        if isinstance(rules, dict):
            rules = json.dumps(rules)
        params = {
            "rules": rules,
        }
        api_path = utils.format_url(
            "/v1/sys/policy/{name}",
            name=name,
        )
        self._adapter.put(api_path, json=params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.delete_policy,
    )
    def delete_policy(self, name):
        self.sys.delete_policy(name=name)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.is_sealed,
    )
    def is_sealed(self):
        return self.sys.is_sealed()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.seal,
    )
    def seal(self):
        self.sys.seal()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.submit_unseal_key,
    )
    def unseal_reset(self):
        return self.sys.submit_unseal_key(reset=True)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.submit_unseal_key,
    )
    def unseal(self, key):
        return self.sys.submit_unseal_key(key=key)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.submit_unseal_keys,
    )
    def unseal_multi(self, keys):
        return self.sys.submit_unseal_keys(keys=keys)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.list_mounted_secrets_engines,
    )
    def list_secret_backends(self):
        return self.sys.list_mounted_secrets_engines()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.enable_secrets_engine,
    )
    def enable_secret_backend(
        self,
        backend_type,
        description=None,
        mount_point=None,
        config=None,
        options=None,
    ):
        return self.sys.enable_secrets_engine(
            backend_type=backend_type,
            path=mount_point,
            description=description,
            config=config,
            options=options,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.tune_mount_configuration,
    )
    def tune_secret_backend(
        self,
        backend_type,
        mount_point=None,
        default_lease_ttl=None,
        max_lease_ttl=None,
        description=None,
        audit_non_hmac_request_keys=None,
        audit_non_hmac_response_keys=None,
        listing_visibility=None,
        passthrough_request_headers=None,
    ):
        if not mount_point:
            mount_point = backend_type

        return self.sys.tune_mount_configuration(
            path=mount_point,
            default_lease_ttl=default_lease_ttl,
            max_lease_ttl=max_lease_ttl,
            description=description,
            audit_non_hmac_request_keys=audit_non_hmac_request_keys,
            audit_non_hmac_response_keys=audit_non_hmac_response_keys,
            listing_visibility=listing_visibility,
            passthrough_request_headers=passthrough_request_headers,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.read_mount_configuration,
    )
    def get_secret_backend_tuning(self, backend_type, mount_point=None):
        """GET /sys/mounts/<mount point>/tune

        :param backend_type: Name of the secret engine. E.g. "aws".
        :type backend_type: str | unicode
        :param mount_point: Alternate argument for backend_type.
        :type mount_point: str | unicode
        :return: The specified mount's configuration.
        :rtype: dict
        """
        if not mount_point:
            mount_point = backend_type
        return self.sys.read_mount_configuration(
            path=mount_point,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.disable_secrets_engine,
    )
    def disable_secret_backend(self, mount_point):
        return self.sys.disable_secrets_engine(
            path=mount_point,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.move_backend,
    )
    def remount_secret_backend(self, from_mount_point, to_mount_point):
        return self.sys.move_backend(
            from_path=from_mount_point,
            to_path=to_mount_point,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.read_lease,
    )
    def read_lease(self, lease_id):
        return self.sys.read_lease(
            lease_id=lease_id,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.renew_lease,
    )
    def renew_secret(self, lease_id, increment=None):
        return self.sys.renew_lease(
            lease_id=lease_id,
            increment=increment,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.revoke_lease,
    )
    def revoke_secret(self, lease_id):
        return self.sys.revoke_lease(
            lease_id=lease_id,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.revoke_lease,
    )
    def revoke_secret_prefix(self, path_prefix):
        self.sys.revoke_prefix(
            prefix=path_prefix,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.rotate_encryption_key,
    )
    def rotate(self):
        self.sys.rotate_encryption_key()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.start_rekey,
    )
    def start_rekey(
        self, secret_shares=5, secret_threshold=3, pgp_keys=None, backup=False
    ):
        return self.sys.start_rekey(
            secret_shares=secret_shares,
            secret_threshold=secret_threshold,
            pgp_keys=pgp_keys,
            backup=backup,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.cancel_rekey,
    )
    def cancel_rekey(self):
        return self.sys.cancel_rekey()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.rekey,
    )
    def rekey(self, key, nonce=None):
        self.sys.rekey(
            key=key,
            nonce=nonce,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.rekey_multi,
    )
    def rekey_multi(self, keys, nonce=None):
        return self.sys.rekey_multi(
            keys=keys,
            nonce=nonce,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.read_backup_keys,
    )
    def get_backed_up_keys(self):
        return self.sys.read_backup_keys()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.is_initialized,
    )
    def is_initialized(self):
        return self.sys.is_initialized()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.initialize,
    )
    def initialize(self, secret_shares=5, secret_threshold=3, pgp_keys=None):
        return self.sys.initialize(
            secret_shares=secret_shares,
            secret_threshold=secret_threshold,
            pgp_keys=pgp_keys,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.start_root_token_generation,
    )
    def start_generate_root(self, key, otp=False):
        params = {}
        if otp:
            params["otp"] = key
        else:
            params["pgp_key"] = key
        return self.sys.start_root_token_generation(**params)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.generate_root,
    )
    def generate_root(self, key, nonce):
        return self.sys.generate_root(
            key=key,
            nonce=nonce,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.cancel_root_generation,
    )
    def cancel_generate_root(self):
        return self.sys.cancel_root_generation().status_code == 204

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.list_auth_methods,
    )
    def list_auth_backends(self):
        return self.sys.list_auth_methods()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.enable_auth_method,
    )
    def enable_auth_backend(
        self,
        backend_type,
        description=None,
        mount_point=None,
        config=None,
        plugin_name=None,
    ):
        return self.sys.enable_auth_method(
            method_type=backend_type,
            description=description,
            config=config,
            path=mount_point,
            plugin_name=plugin_name,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.tune_auth_method,
    )
    def tune_auth_backend(
        self,
        backend_type,
        mount_point=None,
        default_lease_ttl=None,
        max_lease_ttl=None,
        description=None,
        audit_non_hmac_request_keys=None,
        audit_non_hmac_response_keys=None,
        listing_visibility="",
        passthrough_request_headers=None,
    ):
        if not mount_point:
            mount_point = backend_type
        return self.sys.tune_auth_method(
            path=mount_point,
            default_lease_ttl=default_lease_ttl,
            max_lease_ttl=max_lease_ttl,
            description=description,
            audit_non_hmac_request_keys=audit_non_hmac_request_keys,
            audit_non_hmac_response_keys=audit_non_hmac_response_keys,
            listing_visibility=listing_visibility,
            passthrough_request_headers=passthrough_request_headers,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.read_auth_method_tuning,
    )
    def get_auth_backend_tuning(self, backend_type, mount_point=None):
        if not mount_point:
            mount_point = backend_type
        return self.sys.read_auth_method_tuning(
            path=mount_point,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.disable_auth_method,
    )
    def disable_auth_backend(self, mount_point):
        return self.sys.disable_auth_method(
            path=mount_point,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.list_enabled_audit_devices,
    )
    def list_audit_backends(self):
        return self.sys.list_enabled_audit_devices()

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.enable_audit_device,
    )
    def enable_audit_backend(
        self, backend_type, description=None, options=None, name=None
    ):
        self.sys.enable_audit_device(
            device_type=backend_type,
            description=description,
            options=options,
            path=name,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.disable_audit_device,
    )
    def disable_audit_backend(self, name):
        self.sys.disable_audit_device(
            path=name,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.SystemBackend.calculate_hash,
    )
    def audit_hash(self, name, input):
        return self.sys.calculate_hash(
            path=name,
            input_to_hash=input,
        )

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=api.auth_methods.Ldap.login,
    )
    def auth_ldap(self, *args, **kwargs):
        return self.auth.ldap.login(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.9.0",
        new_method=api.auth_methods.Gcp.login,
    )
    def auth_gcp(self, *args, **kwargs):
        return self.auth.gcp.login(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=api.auth_methods.Github.login,
    )
    def auth_github(self, *args, **kwargs):
        return self.auth.github.login(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.close,
    )
    def close(self):
        return self._adapter.close()

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.get,
    )
    def _get(self, *args, **kwargs):
        return self._adapter.get(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.post,
    )
    def _post(self, *args, **kwargs):
        return self._adapter.post(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.put,
    )
    def _put(self, *args, **kwargs):
        return self._adapter.put(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.delete,
    )
    def _delete(self, *args, **kwargs):
        return self._adapter.delete(*args, **kwargs)

    @staticmethod
    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.urljoin,
    )
    def urljoin(*args):
        return adapters.Request.urljoin(*args)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=adapters.Request.request,
    )
    def __request(self, *args, **kwargs):
        return self._adapter.request(*args, **kwargs)

    @utils.deprecated_method(
        to_be_removed_in_version="0.8.0",
        new_method=utils.raise_for_error,
    )
    def __raise_error(self, *args, **kwargs):
        utils.raise_for_error(*args, **kwargs)



"""Vault secrets engines endpoints"""
from hvac.api.secrets_engines.active_directory import ActiveDirectory
from hvac.api.secrets_engines.aws import Aws
from hvac.api.secrets_engines.azure import Azure
from hvac.api.secrets_engines.consul import Consul
from hvac.api.secrets_engines.database import Database
from hvac.api.secrets_engines.gcp import Gcp
from hvac.api.secrets_engines.identity import Identity
from hvac.api.secrets_engines.kv import Kv
from hvac.api.secrets_engines.kv_v1 import KvV1
from hvac.api.secrets_engines.kv_v2 import KvV2
from hvac.api.secrets_engines.pki import Pki
from hvac.api.secrets_engines.rabbitmq import RabbitMQ
from hvac.api.secrets_engines.transform import Transform
from hvac.api.secrets_engines.transit import Transit
from hvac.api.vault_api_category import VaultApiCategory

__all__ = (
    "Aws",
    "Azure",
    "Gcp",
    "ActiveDirectory",
    "Identity",
    "Kv",
    "KvV1",
    "KvV2",
    "Pki",
    "Transform",
    "Transit",
    "SecretsEngines",
    "Database",
    "RabbitMQ",
)


class SecretsEngines(VaultApiCategory):
    """Secrets Engines."""

    implemented_classes = [
        Aws,
        Azure,
        Gcp,
        ActiveDirectory,
        Identity,
        Kv,
        Pki,
        Transform,
        Transit,
        Database,
        Consul,
        RabbitMQ,
    ]
    unimplemented_classes = [
        "AliCloud",
        "Azure",
        "GcpKms",
        "Nomad",
        "Ssh",
        "TOTP",
        "Cassandra",
        "MongoDb",
        "Mssql",
        "MySql",
        "PostgreSql",
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Transform secrets engine methods module."""
from hvac import utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "transform"


class Transform(VaultApiBase):
    """Transform Secrets Engine (API).

    Reference: https://www.vaultproject.io/api-docs/secret/transform
    """

    def create_or_update_role(
        self, name, transformations, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Creates or update the role with the given name.

        If a role with the name does not exist, it will be created. If the role exists, it will be
        updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/role/:name.

        :param name: the name of the role to create. This is part of the request URL.
        :type name: str | unicode
        :param transformations: Specifies the transformations that can be used with this role.
            At least one transformation is required.
        :type transformations: list
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_role request.
        :rtype: requests.Response
        """
        params = {
            "transformations": transformations,
        }
        api_path = "/v1/{mount_point}/role/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Query an existing role by the given name.

        Supported methods:
            GET: /{mount_point}/role/:name.

        :param name: the name of the role to read. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_role request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/role/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_roles(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing roles in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/role.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_roles request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/role".format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing role by the given name.

        Supported methods:
            DELETE: /{mount_point}/role/:name.

        :param name: the name of the role to delete. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_role request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/role/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_or_update_transformation(
        self,
        name,
        transform_type,
        template,
        tweak_source="supplied",
        masking_character="*",
        allowed_roles=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create or update a transformation with the given name.

        If a transformation with the name does not exist, it will be created. If the
        transformation exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/transformation/:name.

        :param name: the name of the transformation to create or update. This is part of
            the request URL.
        :type name: str | unicode
        :param transform_type: Specifies the type of transformation to perform.
            The types currently supported by this backend are fpe and masking.
            This value cannot be modified by an update operation after creation.
        :type transform_type: str | unicode
        :param template: the template name to use for matching value on encode and decode
            operations when using this transformation.
        :type template: str | unicode
        :param tweak_source: Only used when the type is FPE.
        :type tweak_source: str | unicode
        :param masking_character: the character to use for masking. If multiple characters are
            provided, only the first one is used and the rest is ignored. Only used when
            the type is masking.
        :type masking_character: str | unicode
        :param allowed_roles: a list of allowed roles that this transformation can be assigned to.
            A role using this transformation must exist in this list in order for
            encode and decode operations to properly function.
        :type allowed_roles: list
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_ation request.
        :rtype: requests.Response
        """
        params = {
            "type": transform_type,
            "template": template,
            "tweak_source": tweak_source,
            "masking_character": masking_character,
        }
        params.update(
            utils.remove_nones(
                {
                    "allowed_roles": allowed_roles,
                }
            )
        )
        api_path = "/v1/{mount_point}/transformation/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def create_or_update_fpe_transformation(
        self,
        name,
        template,
        tweak_source="supplied",
        allowed_roles=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Creates or update an FPE transformation with the given name.

        If a transformation with the name does not exist, it will be created. If the transformation exists, it will be
        updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/transformations/fpe/:name.


        :param name: The name of the transformation to create or update. This is part of
            the request URL.
        :type name: str
        :param template: The template name to use for matching value on encode and decode
            operations when using this transformation.
        :type template: str
        :param tweak_source: Specifies the source of where the tweak value comes from. Valid sources are:
            supplied, generated, and internal.
        :type tweak_source: str
        :param allowed_roles: A list of allowed roles that this transformation can be assigned to.
            A role using this transformation must exist in this list in order for
            encode and decode operations to properly function.
        :type allowed_roles: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create_or_update_fpe_transformation request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "template": template,
                "tweak_source": tweak_source,
                "allowed_roles": allowed_roles,
            }
        )
        api_path = "/v1/{mount_point}/transformations/fpe/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def create_or_update_masking_transformation(
        self,
        name,
        template,
        masking_character="*",
        allowed_roles=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Creates or update a masking transformation with the given name. If a
        transformation with the name does not exist, it will be created. If the
        transformation exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/transformations/masking/:name.


        :param name: The name of the transformation to create or update. This is part of
            the request URL.
        :type name: str
        :param template: The template name to use for matching value on encode and decode
            operations when using this transformation.
        :type template: str
        :param masking_character: The character to use for masking. If multiple characters are
            provided, only the first one is used and the rest is ignored. Only used when
            the type is masking.
        :type masking_character: str
        :param allowed_roles: A list of allowed roles that this transformation can be assigned to.
            A role using this transformation must exist in this list in order for
            encode and decode operations to properly function.
        :type allowed_roles: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create_or_update_masking_transformation request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "template": template,
                "masking_character": masking_character,
                "allowed_roles": allowed_roles,
            }
        )
        api_path = "/v1/{mount_point}/transformations/masking/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def create_or_update_tokenization_transformation(
        self,
        name,
        max_ttl=0,
        mapping_mode="default",
        allowed_roles=None,
        stores=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """
        This endpoint creates or updates a tokenization transformation with the given name. If a
        transformation with the name does not exist, it will be created. If the
        transformation exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/transformations/tokenization/:name.

        :param max_ttl: The maximum TTL of a token. If 0 or unspecified, tokens may have no expiration.
        :type max_ttl: str
        :param mapping_mode: Specifies the mapping mode for stored tokenization values.

            * `default` is strongly recommended for highest security
            * `exportable` exportable allows for all plaintexts to be decoded via the export-decoded endpoint in an emergency.

        :type mapping_mode: str
        :param allowed_roles: aAlist of allowed roles that this transformation can be assigned to.
            A role using this transformation must exist in this list in order for
            encode and decode operations to properly function.
        :type allowed_roles: list
        :param stores: list of tokenization stores to use for tokenization state. Vault's
            internal storage is used by default.
        :type stores: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create_or_update_tokenization_transformation request.
        :rtype: requests.Response
        """
        if stores is None:
            stores = ["builtin/internal"]
        params = utils.remove_nones(
            {
                "max_ttl": max_ttl,
                "mapping_mode": mapping_mode,
                "allowed_roles": allowed_roles,
                "stores": stores,
            }
        )
        api_path = "/v1/{mount_point}/transformations/tokenization/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_transformation(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Query an existing transformation by the given name.

        Supported methods:
            GET: /{mount_point}/transformation/:name.

        :param name: Specifies the name of the role to read.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_ation request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/transformation/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_transformations(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing transformations in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/transformation.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_ation request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/transformation".format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_transformation(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing transformation by the given name.

        Supported methods:
            DELETE: /{mount_point}/transformation/:name.

        :param name: the name of the transformation to delete. This is part of the
            request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_ation request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/transformation/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_or_update_template(
        self, name, template_type, pattern, alphabet, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Creates or update a template with the given name.

        If a template with the name does not exist, it will be created. If the
        template exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/template/:name.

        :param name: the name of the template to create.
        :type name: str | unicode
        :param template_type: Specifies the type of pattern matching to perform.
            The ony type currently supported by this backend is regex.
        :type template_type: str | unicode
        :param pattern: the pattern used to match a particular value. For regex type
            matching, capture group determines the set of character that should be matched
            against. Any matches outside of capture groups are retained
            post-transformation.
        :type pattern: str | unicode
        :param alphabet: the name of the alphabet to use when this template is used for FPE
            encoding and decoding operations.
        :type alphabet: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_template request.
        :rtype: requests.Response
        """
        params = {
            "type": template_type,
            "pattern": pattern,
            "alphabet": alphabet,
        }
        api_path = "/v1/{mount_point}/template/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_template(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Query an existing template by the given name.

        Supported methods:
            GET: /{mount_point}/template/:name.

        :param name: Specifies the name of the role to read.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_template request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/template/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_templates(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing templates in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/transformation.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_template request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/template".format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_template(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing template by the given name.

        Supported methods:
            DELETE: /{mount_point}/template/:name.

        :param name: the name of the template to delete. This is part of the
            request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_template request.
        :rtype: requests.Response
        """
        params = {
            "name": name,
        }
        api_path = "/v1/{mount_point}/template/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
            json=params,
        )

    def create_or_update_alphabet(
        self, name, alphabet, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Create or update an alphabet with the given name.

        If an alphabet with the name does not exist, it will be created. If the
        alphabet exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/alphabet/:name.

        :param name: Specifies the name of the transformation alphabet to create.
        :type name: str | unicode
        :param alphabet: the set of characters that can exist within the provided value
            and the encoded or decoded value for a FPE transformation.
        :type alphabet: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_alphabet request.
        :rtype: requests.Response
        """
        params = {
            "alphabet": alphabet,
        }
        api_path = "/v1/{mount_point}/alphabet/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_alphabet(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Queries an existing alphabet by the given name.

        Supported methods:
            GET: /{mount_point}/alphabet/:name.


        :param name: the name of the alphabet to delete. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_alphabet request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/alphabet/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_alphabets(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing alphabets in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/alphabet.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_alphabets request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/alphabet".format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_alphabet(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing alphabet by the given name.

        Supported methods:
            DELETE: /{mount_point}/alphabet/:name.

        :param name: the name of the alphabet to delete. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_alphabet request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/alphabet/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_or_update_tokenization_store(
        self,
        name,
        driver,
        connection_string,
        username=None,
        password=None,
        type="sql",
        supported_transformations=None,
        schema="public",
        max_open_connections=4,
        max_idle_connections=4,
        max_connection_lifetime=0,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create or update a storage configuration for use with tokenization.
        The database user configured here should only have permission to SELECT, INSERT, and UPDATE rows in the tables.

        Supported methods:
            POST: /{mount_point}/store/:name.

        :param name: The name of the store to create or update.
        :type name: str
        :param type: Specifies the type of store. Currently only `sql` is supported.
        :type type: str
        :param driver: Specifies the database driver to use, and thus which SQL database type.
            Currently the supported options are `postgres` or `mysql`
        :type driver: str
        :param supported_transformations: The types of transformations this store can host. Currently only `tokenization` is supported.
        :type supported_transformations: list(str)
        :param connection_string: database connection string with template slots for username and password that
            Vault will use for locating and connecting to a database.  Each
            database driver type has a different syntax for its connection strings.
        :type connection_string: str
        :param username: username value to use when connecting to the database.
        :type username: str
        :param password: password value to use when connecting to the database.
        :type password: str
        :param schema: schema within the database to expect tokenization state tables.
        :type schema: str
        :param max_open_connections: maximum number of connections to the database at any given time.
        :type max_open_connections: int
        :param max_idle_connections: maximum number of idle connections to the database at any given time.
        :type max_idle_connections: int
        :param max_connection_lifetime: means no limit.
        :type max_connection_lifetime: duration
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create_or_update_tokenization_store request.
        :rtype: requests.Response
        """
        if supported_transformations is None:
            supported_transformations = ["tokenization"]
        params = utils.remove_nones(
            {
                "type": type,
                "driver": driver,
                "supported_transformations:": supported_transformations,
                "connection_string": connection_string,
                "username": username,
                "password": password,
                "schema": schema,
                "max_open_connections": max_open_connections,
                "max_idle_connections": max_idle_connections,
                "max_connection_lifetime": max_connection_lifetime,
            }
        )
        api_path = "/v1/{mount_point}/store/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def encode(
        self,
        role_name,
        value=None,
        transformation=None,
        tweak=None,
        batch_input=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Encode the provided value using a named role.

        Supported methods:
            POST: /{mount_point}/encode/:role_name.

        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str | unicode
        :param value: the value to be encoded.
        :type value: str | unicode
        :param transformation: the transformation within the role that should be used for this
            encode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str | unicode
        :param tweak: the tweak source.
        :type tweak: str | unicode
        :param batch_input: a list of items to be encoded in a single batch. When this
            parameter is set, the 'value', 'transformation' and 'tweak' parameters are
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: list
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the encode request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "value": value,
                "transformation": transformation,
                "tweak": tweak,
                "batch_input": batch_input,
            }
        )
        api_path = "/v1/{mount_point}/encode/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def decode(
        self,
        role_name,
        value=None,
        transformation=None,
        tweak=None,
        batch_input=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Decode the provided value using a named role.

        Supported methods:
            POST: /{mount_point}/decode/:role_name.

        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str | unicode
        :param value: the value to be decoded.
        :type value: str | unicode
        :param transformation: the transformation within the role that should be used for this
            decode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str | unicode
        :param tweak: the tweak source.
        :type tweak: str | unicode
        :param batch_input: a list of items to be decoded in a single batch. When this
            parameter is set, the 'value', 'transformation' and 'tweak' parameters are
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: array<object>
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the decode request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "value": value,
                "transformation": transformation,
                "tweak": tweak,
                "batch_input": batch_input,
            }
        )
        api_path = "/v1/{mount_point}/decode/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def validate_token(
        self,
        role_name,
        value,
        transformation,
        batch_input=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Determine if a provided tokenized value is valid and unexpired.
        Only valid for tokenization transformations.

        Supported methods:
            POST: /{mount_point}/validate/:role_name.


        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str
        :param value: the token for which to check validity.
        :type value: str
        :param transformation: the transformation within the role that should be used for this
            decode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str
        :param batch_input: a list of items to be decoded in a single batch. When this
            parameter is set, the 'value' parameter is
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the validate_token request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "value": value,
                "transformation": transformation,
                "batch_input": batch_input,
            }
        )
        api_path = "/v1/{mount_point}/validate/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def check_tokenization(
        self,
        role_name,
        value,
        transformation,
        batch_input=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Determine if a provided plaintext value has an valid, unexpired tokenized value.
        Note that this cannot return the token, just confirm that a
        tokenized value exists. This endpoint is only valid for tokenization
        transformations.

        Supported methods:
            POST: /{mount_point}/tokenized/:role_name.


        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str
        :param value: the token to test for whether it has a valid tokenization.
        :type value: str
        :param transformation: the transformation within the role that should be used for this
            decode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str
        :param batch_input: a list of items to be decoded in a single batch. When this
            parameter is set, the 'value' parameter is
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the check_tokenization request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "value": value,
                "transformation": transformation,
                "batch_input": batch_input,
            }
        )
        api_path = "/v1/{mount_point}/tokenized/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def retrieve_token_metadata(
        self,
        role_name,
        value,
        transformation,
        batch_input=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """
        This endpoint retrieves metadata for a tokenized value using a named role.
        Only valid for tokenization transformations.

        Supported methods:
            POST: /{mount_point}/metadata/:role_name.


        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str
        :param value: the token for which to retrieve metadata.
        :type value: str
        :param transformation: the transformation within the role that should be used for this
            decode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str
        :param batch_input: a list of items to be decoded in a single batch. When this
            parameter is set, the 'value' parameter is
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the retrieve_token_metadata request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "value": value,
                "transformation": transformation,
                "batch_input": batch_input,
            }
        )
        api_path = "/v1/{mount_point}/metadata/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def snapshot_tokenization_state(
        self, name, limit=1000, continuation="", mount_point=DEFAULT_MOUNT_POINT
    ):
        """
        This endpoint starts or continues retrieving a snapshot of the stored
        state of a tokenization transform.  This state is protected as it is
        in the underlying store, and so is safe for storage or transport.  Snapshots
        may be used for backup purposes or to migrate from one store to another.
        If more than one store is configured for a tokenization transform, the
        snapshot data contains the contents of the first store.

        Supported methods:
            POST: /{mount_point}/transformations/tokenization/snapshot/:name.


        :param name: the name of the transformation to snapshot.
        :type name: str
        :param limit: maximum number of tokenized value states to return on this call.
        :type limit: int
        :param continuation: absent or empty, a new snapshot is started.  If present, the
            snapshot should continue at the next available value.
        :type continuation: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the snapshot_tokenization_state request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "limit": limit,
                "continuation": continuation,
            }
        )
        api_path = (
            "/v1/{mount_point}/transformations/tokenization/snapshot/{name}".format(
                mount_point=mount_point,
                name=name,
            )
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def restore_tokenization_state(self, name, values, mount_point=DEFAULT_MOUNT_POINT):
        """
        This endpoint restores previously snapshotted tokenization state values
        to the underlying store(s) of a tokenization transform.  Calls to this
        endpoint are idempotent, so multiple outputs from a snapshot run can
        be applied via restore in any order and duplicates will not cause a problem.

        Supported methods:
            POST: /{mount_point}/transformations/tokenization/restore/:name.


        :param name: the name of the transformation to restore.
        :type name: str
        :param values: number of tokenization state values from a previous snapshot call.
        :type values: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the restore_tokenization_state request.
        :rtype: requests.Response
        """
        params = {
            "values": values,
        }
        api_path = (
            "/v1/{mount_point}/transformations/tokenization/restore/{name}".format(
                mount_point=mount_point,
                name=name,
            )
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def export_decoded_tokenization_state(
        self, name, limit=1000, continuation="", mount_point=DEFAULT_MOUNT_POINT
    ):
        """Start or continue retrieving an export of tokenization state, including the tokens and their decoded values.
        This call is only supported on tokenization stores configured with the exportable mapping mode.
        Refer to the Tokenization documentation for when to use the exportable mapping mode.
        Decoded values are in Base64 representation.

        Supported methods:
            POST: /{mount_point}/transformations/tokenization/export-decoded/:name.


        :param name: the name of the transformation to export.
        :type name: str
        :param limit: maximum number of tokenized value states to return on this call.
        :type limit: int
        :param continuation: absent or empty, a new export is started.  If present, the
            export should continue at the next available value.
        :type continuation: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the export_decoded_tokenization_state request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "limit": limit,
                "continuation": continuation,
            }
        )
        api_path = "/v1/{mount_point}/transformations/tokenization/export-decoded/{name}".format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def rotate_tokenization_key(self, transform_name, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the version of the named key.
        After rotation, new requests will be encoded with the new version of the key.

        Supported methods:
            POST: /{mount_point}/tokenization/keys/{transform_name}/rotate.


        :param transform_name: the transform name to use for this operation. This is specified as part
            of the URL.
        :type transform_name: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the rotate_tokenization_key request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/tokenization/keys/{transform_name}/rotate".format(
            mount_point=mount_point,
            transform_name=transform_name,
        )
        return self._adapter.post(
            url=api_path,
        )

    def update_tokenization_key_config(
        self, transform_name, min_decryption_version, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Allow the minimum key version to be set for decode operations.
        Only valid for tokenization transformations.

        Supported methods:
            POST: /{mount_point}/tokenization/keys/{transform_name}/config.


        :param transform_name: the transform name to use for this operation. This is specified as part
            of the URL.
        :type transform_name: str
        :param min_decryption_version: the minimum key version that vault can use to decode values for the
            corresponding transform.
        :type min_decryption_version: int
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the update_tokenization_key_config request.
        :rtype: requests.Response
        """
        params = {
            "transform_name": transform_name,
            "min_decryption_version": min_decryption_version,
        }
        api_path = "/v1/{mount_point}/tokenization/keys/{transform_name}/config".format(
            mount_point=mount_point,
            transform_name=transform_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def list_tokenization_key_configuration(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all tokenization keys.
        Only valid for tokenization transformations.

        Supported methods:
            LIST: /{mount_point}/tokenization/keys/.

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the list_tokenization_key_configuration request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/tokenization/keys/".format(
            mount_point=mount_point,
        )
        return self._adapter.list(
            url=api_path,
        )

    def read_tokenization_key_configuration(
        self, transform_name, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Read tokenization key configuration for a particular transform.
        Only valid for tokenization transformations.

        Supported methods:
            GET: /{mount_point}/tokenization/keys/:{mount_point}_name.


        :param transform_name: the transform name to use for this operation. This is specified as part
            of the URL.
        :type transform_name: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the read_tokenization_key_configuration request.
        :rtype: requests.Response
        """
        api_path = "/v1/{mount_point}/tokenization/keys/{transform_name}".format(
            mount_point=mount_point,
            transform_name=transform_name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def trim_tokenization_key_version(
        self, transform_name, min_available_version, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Trim older key versions setting a minimum version for the keyring.
        Once trimmed, previous versions of the key cannot be recovered.

        Supported methods:
            POST: /{mount_point}/tokenization/keys/{transform_name}/trim.


        :param transform_name: the transform name to use for this operation. This is specified as part
            of the URL.
        :type transform_name: str
        :param min_available_version:
        :type min_available_version: int
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the trim_tokenization_key_version request.
        :rtype: requests.Response
        """
        params = {
            "min_available_version": min_available_version,
        }
        api_path = "/v1/{mount_point}/tokenization/keys/{transform_name}/trim".format(
            mount_point=mount_point,
            transform_name=transform_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )



"""Collection of Vault API endpoint classes."""
from hvac.api.auth_methods import AuthMethods
from hvac.api.secrets_engines import SecretsEngines
from hvac.api.system_backend import SystemBackend
from hvac.api.vault_api_base import VaultApiBase
from hvac.api.vault_api_category import VaultApiCategory

__all__ = (
    "AuthMethods",
    "SecretsEngines",
    "SystemBackend",
    "VaultApiBase",
    "VaultApiCategory",
)
